#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_batchaccountpool
version_added: "3.0.0"
short_description: Manages a Batch Account Pool on Azure
description:
    - Create, update and delete instance of Azure Batch Account Pool.

options:
    resource_group:
        description:
            - The name of the resource group in which to create the Batch Account Pool.
        required: true
        type: str
    batch_account_name:
        description:
            - The name of the Batch Account.
        required: true
        type: str
    name:
        description:
            - The name of the Batch Account Pool.
        required: true
        type: str
    display_name:
        description:
            - The display name for the pool.
        type: str
    identity:
        description:
            - The type of identity used for the Batch Pool.
            - If the pool identity is updated during update an existing pool.
            - Only the new vms which are created after the pool shrinks to 0 will have the updated identities.
        type: dict
        suboptions:
            type:
                description:
                    - The type of identity used for the Batch Pool.
                required: true
                type: str
                choices:
                    - None
                    - UserAssigned
            user_assigned_identities:
                description:
                    - The list of user identities associated with the Batch pool.
                    - The key is the identity's ID and value contains 'principal_id' and 'client_id'.
                type: dict
    vm_size:
        description:
            - For information about available sizes of virtual machines for Cloud Services Pools.
            - Batch supports all Cloud Services VM sizes except ExtraSmall, A1V2 and A2V2,
            - Batch supports all Azure VM sizes except STANDARD_A0 and those with premium storage (STANDARD_GS, STANDARD_DS, and STANDARD_DSV2 series).
        type: str
    deployment_configuration:
        description:
            - Using CloudServiceConfiguration specifies that the nodes should be creating using Azure Cloud Services (PaaS).
            - Using VirtualMachineConfiguration uses Azure Virtual Machines (IaaS).
        type: dict
        suboptions:
            cloud_service_configuration:
                description:
                    - This property and virtualMachineConfiguration are mutually exclusive and one of the properties must be specified.
                    - This property cannot be specified if the Batch account was created with its poolAllocationMode property set to 'UserSubscription'.
                type: dict
                suboptions:
                    os_family:
                        description:
                            - I(os_family=2) equivalent to Windows Server 2008 R2 SP1.
                            - I(os_family=3) equivalent to Windows Server 2012.
                            - I(os_family=4) equivalent to Windows Server 2012 R2.
                            - I(os_family=5) equivalent to Windows Server 2016.
                            - I(os_family=6) equivalent to Windows Server 2019.
                        type: str
                        default: '*'
                    os_version:
                        description:
                            - The default value is C(*) which specifies the latest operating system version for the specified OS family.
                        type: str
            virtual_machine_configuration:
                description:
                    - This property and cloudServiceConfiguration are mutually exclusive and one of the properties must be specified.
                type: dict
                suboptions:
                    image_reference:
                        description:
                            - A reference to an Azure Virtual Machines Marketplace image or the zure Image resource of a custom Virtual Machine.
                            - To get the list of all imageReferences verified by Azure Batch, see the 'List supported node agent SKUs' operation.
                        type: dict
                        required: true
                        suboptions:
                            publisher:
                                description:
                                    - For example, C(Canonical) or C(MicrosoftWindowsServer).
                                type: str
                            offer:
                                description:
                                    - For example, C(UbuntuServer) or C(WindowsServer).
                                type: str
                            sku:
                                description:
                                    - For example, C(18.04-LTS) or C(2022-datacenter).
                                type: str
                            version:
                                description:
                                    - A value of 'latest' can be specified to select the latest version of an image.
                                    - If omitted, the default is C(latest).
                                type: str
                            id:
                                description:
                                    - This property is mutually exclusive with other properties.
                                    - The Azure Compute Gallery Image must have replicas in the same region as the Azure Batch account.
                                    - Sample as C('/subscriptions/{sub_Id}/resourceGroups/{resourceGroup}/providers/Microsoft.Compute/images/{imageName}).
                                type: str
                    node_agent_sku_id:
                        description:
                            - The Batch node agent is a program that runs on each node in the pool.
                            - Provides the command-and-control interface between the node and the Batch service.
                            - There are different implementations of the node agent, known as SKUs, for different operating systems.
                            - You must specify a node agent SKU which matches the selected image reference.
                            - To get the list of supported node agent SKUs along with their list of verified image references.
                        type: str
                        required: true
                    windows_configuration:
                        description:
                            - This property must not be specified if the imageReference specifies a Linux OS image.
                        type: dict
                        suboptions:
                            enable_automatic_updates:
                                description:
                                    - If omitted, the default value is C(true).
                                type: bool
                                default: true
                    data_disks:
                        description:
                            - This property must be specified if the compute nodes in the pool need to have empty data disks attached to them.
                        type: list
                        elements: dict
                        suboptions:
                            lun:
                                description:
                                    - The lun is used to uniquely identify each data disk.
                                    - If attaching multiple disks, each should have a distinct lun.
                                    - The value must be between 0 and 63, inclusive.
                                required: true
                                type: int
                            caching:
                                description:
                                    - The caching mode for the disk
                                type: str
                                choices:
                                    - None
                                    - ReadOnly
                                    - ReadWrite
                            disk_size_gb:
                                description:
                                    - The initial disk size in GB when creating new data disk.
                                type: int
                                required: true
                            storage_account_type:
                                description:
                                    - The data disk type.
                                    - C(Standard_LRS) for the data disk should use standard locally redundant storage.
                                    - C(Premium_LRS) for the he data disk should use premium locally redundant storage.
                                type: str
                                default: Standard_LRS
                                choices:
                                    - Standard_LRS
                                    - Premium_LRS
                                    - StandardSSD_LRS
                    license_type:
                        description:
                            - This only applies to images that contain the Windows operating system.
                            - Should only be used when you hold valid on-premises licenses for the nodes which will be deployed.
                            - C(Windows_Server), The on-premises license is for Windows Server.
                            - C(Windows_Client), The on-premises license is for Windows Client.
                        type: str
                    container_configuration:
                        description:
                            - If specified, setup is performed on each node in the pool to allow tasks to run in containers.
                            - All regular tasks and job manager tasks run on this pool must specify the containerSettings property.
                            - All other tasks may specify it.
                        type: dict
                        suboptions:
                            type:
                                description:
                                    - The container technology to be used.
                                required: true
                                type: str
                                choices:
                                    - DockerCompatible
                                    - CriCompatible
                            container_image_names:
                                description:
                                    - This is the full image reference, as would be specified to "docker pull".
                                    - "An image will be sourced from the default Docker registry
                                      unless the image is fully qualified with an alternative registry."
                                type: list
                                elements: str
                            container_registries:
                                description:
                                    - "If any images must be downloaded from a private registry which requires credentials,
                                      then those credentials must be provided here."
                                type: list
                                elements: dict
                                suboptions:
                                    user_name:
                                        description:
                                            - The user name to log into the registry server.
                                        type: str
                                    password:
                                        description:
                                            - The password to log into the registry server.
                                        type: str
                                    registry_server:
                                        description:
                                            - If omitted, the default is C(docker.io).
                                        type: str
                                        default: 'docker.io'
                                    identity_reference:
                                        description:
                                            - The reference to a user assigned identity associated with the Batch pool which a compute node will use.
                                        type: dict
                                        suboptions:
                                            resource_id:
                                                description:
                                                    - The ARM resource id of the user assigned identity.
                                                type: str
                    disk_encryption_configuration:
                        description:
                            - If specified, encryption is performed on each node in the pool during node provisioning.
                        type: dict
                        suboptions:
                            targets:
                                description:
                                    - On Linux pool, only C(TemporaryDisk) is supported.
                                    - on Windows pool, C(OsDisk) and C(TemporaryDisk) must be specified.
                                type: list
                                elements: str
                                choices:
                                    - TemporaryDisk
                                    - OsDisk
                    node_placement_configuration:
                        description:
                            - This configuration will specify rules on how nodes in the pool during node provisioning.
                        type: dict
                        suboptions:
                            policy:
                                description:
                                    - Allocation policy used by Batch Service to provision the nodes.
                                    - If not specified, Batch will use the regional policy.
                                type: str
                                choices:
                                    - Regional
                                    - Zonal
                    extensions:
                        description:
                            - If specified, the extensions mentioned in this configuration will be installed on each node.
                        type: list
                        elements: dict
                        suboptions:
                            name:
                                description:
                                    - The name of the virtual machine extension.
                                required: true
                                type: str
                            publisher:
                                description:
                                    - The name of the extension handler publisher.
                                required: true
                                type: str
                            type:
                                description:
                                    - The type of the extensions.
                                type: str
                                required: true
                            type_handler_version:
                                description:
                                    - The version of script handler.
                                type: str
                            auto_upgrade_minor_version:
                                description:
                                    - Indicates whether the extension should use a newer minor version if one is available at deployment time.
                                    - Once deployed, the extension will not upgrade minor versions unless redeployed, even with this property set to true.
                                type: bool
                            enable_automatic_upgrade:
                                description:
                                    - "Indicates whether the extension should be automatically upgraded by the platform
                                      if there is a newer version of the extension available."
                                type: bool
                            settings:
                                description:
                                    - JSON formatted public settings for the extension.
                                type: json
                            protected_settings:
                                description:
                                    - The extension can contain either protectedSettings or protectedSettingsFromKeyVault or no protected settings at all.
                                type: json
                            provision_after_extensions:
                                description:
                                    - Collection of extension names after which this extension needs to be provisioned.
                                type: list
                                elements: str
                    os_disk:
                        description:
                            - Contains configuration for ephemeral OSDisk settings.
                        type: dict
                        suboptions:
                            ephemeral_os_disk_settings:
                                description:
                                    - Specifies the ephemeral Disk Settings for the operating system disk used by the virtual machine.
                                type: dict
                                suboptions:
                                    placement:
                                        description:
                                            - This property can be used by user in the request to choose which location the operating system should be in.
                                        type: str
                            caching:
                                description:
                                    - The type of caching to enable for the disk.
                                type: str
                                choices:
                                    - None
                                    - ReadOnly
                                    - ReadWrite
                            managed_disk:
                                description:
                                    - The data disk config.
                                type: dict
                                suboptions:
                                    storage_account_type:
                                        description:
                                            - The storage account type for use in creating data disks or OSdisk.
                                        type: str
                                        choices:
                                            - Standard_LRS
                                            - Premium_LRS
                                            - StandardSSD_LRS
                            disk_size_gb:
                                description:
                                    - The initial disk size in GB when creating new OS disk.
                                type: int
                            write_accelerator_enabled:
                                description:
                                    -  Specifies whether writeAccelerator should be enabled or disabled on the disk.
                                type: bool
                    security_profile:
                        description:
                            - Specifies the security profile settings for the virtual machine or virtual machine scale set.
                        type: dict
                        suboptions:
                            security_type:
                                description:
                                    - Specifies the SecurityType of the virtual machine.
                                    - It has to be set to any specified value to enable UefiSettings.
                                type: str
                                default: trustedLaunch
                            encryption_at_host:
                                description:
                                    - "This property can be used by user in the request to enable or disable the Host Encryption
                                      for the virtual machine or virtual machine scale set."
                                    - This willenable the encryption for all the disks including Resource/Temp disk at host itself.
                                type: bool
                            uefi_settings:
                                description:
                                    - Specifies the security settings like secure boot and vTPM used while creating the virtual machine.
                                type: dict
                                suboptions:
                                    secure_boot_enabled:
                                        description:
                                            - Specifies whether secure boot should be enabled on the virtual machine.
                                        type: bool
                                    v_tpm_enabled:
                                        description:
                                            - Specifies whether vTPM should be enabled on the virtual machine.
                                        type: bool
                    service_artifact_reference:
                        description:
                            - The service artifact reference ID.
                            - "Such as C(/subscriptions/{subId}/resourceGroups/{testRG}/providers/Microsoft.Compute/galleries
                              /{gName}/serviceArtifacts/{ArtName}/vmArtifactsProfiles/{ProfilesName})."
                        type: dict
                        suboptions:
                            id:
                                description:
                                    - The service artifact reference ID of the vmArtifactsProfiles.
                                type: str
    scale_settings:
        description:
            - Defines the desired size of the pool.
            - "This can either be 'fixedScale' where the requested targetDedicatedNodes is specified,
              or 'autoScale' which defines a formula which is periodically reevaluated."
            - If this property is not specified, the pool will have a fixed scale with 0 targetDedicatedNodes.
        type: dict
        suboptions:
            fixed_scale:
                description:
                    - This property and autoScale are mutually exclusive and one of the properties must be specified.
                type: dict
                suboptions:
                    resize_timeout:
                        description:
                            - The default value is 15 minutes C(P15M).
                            - Timeout values use ISO 8601 format. For example, use PT10M for 10 minutes.
                            - The minimum value is 5 minutes.
                            - If you specify a value less than 5 minutes, the Batch service rejects the request with an error.
                        type: str
                        default: P15M
                    target_dedicated_nodes:
                        description:
                            -  At least one of targetDedicatedNodes, targetLowPriorityNodes must be set.
                        type: int
                    target_low_priority_nodes:
                        description:
                            - At least one of targetDedicatedNodes, targetLowPriorityNodes must be set.
                        type: int
                    node_deallocation_option:
                        description:
                            -  If omitted, the default value is C(Requeue).
                        type: str
                        choices:
                            - Requeue
                            - Terminate
                            - TaskCompletion
                            - RetainedData
            auto_scale:
                description:
                    - This property and fixedScale are mutually exclusive and one of the properties must be specified. must be specified.
                type: dict
                suboptions:
                    formula:
                        description:
                            - A formula for the desired number of compute nodes in the pool.
                        type: str
                        required: true
                    evaluation_interval:
                        description:
                            - If omitted, the default value is 15 minutes (PT15M).
                        type: str
                        default: P15M
    inter_node_communication:
        description:
            - This imposes restrictions on which nodes can be assigned to the pool.
            - Enabling this value can reduce the chance of the requested number of nodes to be allocated in the pool.
            - If not specified, this value defaults to C(Disabled).
        type: str
        choices:
            - Enabled
            - Disabled
    network_configuration:
        description:
            - The network configuration for a pool.
        type: dict
        suboptions:
            subnet_id:
                description:
                    - The virtual network must be in the same region and subscription as the Azure Batch account.
                    - The specified subnet should have enough free IP addresses to accommodate the number of nodes in the pool.
                    - If the subnet doesn't have enough free IP addresses, the pool will partially allocate compute nodes and a resize error will occur.
                type: str
            dynamic_vnet_assignment_scope:
                description:
                    - The scope of dynamic vnet assignment.
                type: str
                choices:
                    - none
                    - job
            endpoint_configuration:
                description:
                    - Pool endpoint configuration is only supported on pools with the virtualMachineConfiguration property.
                type: dict
                suboptions:
                    inbound_nat_pools:
                        description:
                            - The maximum number of inbound NAT pools per Batch pool is 5.
                            - If the maximum number of inbound NAT pools is exceeded the request fails with HTTP status code 400.
                        type: list
                        elements: dict
                        suboptions:
                            name:
                                description:
                                    - The name must be unique within a Batch pool.
                                    - The name contain letters, numbers, underscores, periods, and hyphens.
                                type: str
                                required: true
                            protocol:
                                description:
                                    - The protocol of the endpoint.
                                type: str
                                required: true
                                choices:
                                    - UDP
                                    - TCP
                            backend_port:
                                description:
                                    - This must be unique within a Batch pool.
                                    - Acceptable values are between 1 amd 65535 except for 22.
                                type: int
                                required: true
                            frontend_port_range_start:
                                description:
                                    - Acceptable values range between 1 and 65534 except ports overlap
                                    - If any reserved or overlapping values are provided the request fails with HTTP status code 400.
                                type: int
                                required: true
                            frontend_port_range_end:
                                description:
                                    - Acceptable values range between 1 and 65534 except ports from 50000 to 55000 which are reserved by the Batch service.
                                    - All ranges within a pool must be distinct and cannot overlap.
                                type: int
                                required: true
                            network_security_group_rules:
                                description:
                                    - The maximum number of rules that can be specified across all the endpoints on a Batch pool is 25.
                                    - If no network security group rules are specified.
                                    - default rule will be created to allow inbound access to the specified backendPort.
                                type: list
                                elements: dict
                                suboptions:
                                    priority:
                                        description:
                                            - Priorities within a pool must be unique and are evaluated in order of priority
                                        type: int
                                        required: true
                                    access:
                                        description:
                                            - The action that should be taken for a specified IP address, subnet range or tags.
                                        type: str
                                        choices:
                                            - Allow
                                            - Deny
                                        required: true
                                    source_address_prefix:
                                        description:
                                            - Valid values are a single IP address (i.e. 10.10.10.10).
                                            - Valid values are a single IP address.
                                        type: str
                                    source_port_ranges:
                                        description:
                                            - Valid values are '*' (for all ports 0 - 65535) or arrays of orts or port ranges (i.e. 100-200).
                                        type: list
                                        elements: str
            public_ip_address_configuration:
                description:
                    - This property is only supported on Pools with the virtualMachineConfiguration property.
                type: dict
                suboptions:
                    provision:
                        description:
                            - The public IP Address configuration's provision.
                        type: str
                        default: BatchManaged
                        choices:
                            - BatchManaged
                            - UserManaged
                            - NoPublicIPAddresses
                    ip_address_ids:
                        description:
                            - "The number of IPs specified here limits the maximum size of the Pool
                              100 dedicated nodes or 100 Spot/low-priority nodes can be allocated for each public IP."
                        type: list
                        elements: str
            enable_accelerated_networking:
                description:
                    - Accelerated networking enables single root I/O virtualization (SR-IOV) to a VM.
                    - Which may lead to improved networking performance.
                type: bool
    task_slots_per_node:
        description:
            - The default value is C(1).
            - The maximum value is the smaller of 4 times the number of cores of the vmSize of the pool or 256.
        type: int
        default: 1
    task_scheduling_policy:
        description:
            - Specifies how tasks should be distributed across compute nodes.
        type: dict
        suboptions:
            node_fill_type:
                description:
                    - How tasks should be distributed across compute nodes.
                type: str
                choices:
                    - Spread
                    - Pack
    user_accounts:
        description:
            - The list of user accounts to be created on each node in the pool.
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name of the user account. Names can contain any Unicode characters up to a maximum length of 20.
                type: str
                required: true
            password:
                description:
                    - The password for the user account.
                type: str
                required: true
            elevation_level:
                description:
                    - C(NonAdmin) - the auto user is a standard user without elevated access.
                    - C(Admin) - The auto user is a user with elevated access and operates with full Administrator permissions.
                type: str
                default: NonAdmin
                choices:
                    - NonAdmin
                    - Admin
            linux_user_configuration:
                description:
                    - Properties used to create a user account on a Linux node.
                type: dict
                suboptions:
                    uid:
                        description:
                            - The uid and gid properties must be specified together or not at all.
                            - If not specified the underlying operating system picks the uid.
                        type: int
                    gid:
                        description:
                            - The uid and gid properties must be specified together or not at all.
                            - If not specified the underlying operating system picks the gid.
                        type: int
                    ssh_private_key:
                        description:
                            - The private key must not be password protected.
                            - "The private key is used to automatically configure asymmetric-key based authentication for SSH between nodes in a Linux pool
                               when the pool's enableInterNodeCommunication property is true."
                            - It does this by placing the key pair into the user's .ssh directory.
                            - If not specified, password-less SSH is not configured between nodes (no modification of the user's .ssh directory is done).
                        type: str
            windows_user_configuration:
                description:
                    - Properties used to create a user account on a Windows node.
                type: dict
                suboptions:
                    login_mode:
                        description:
                            - Specifies login mode for the user.
                            - The default value for VirtualMachineConfiguration pools is interactive mode and for CloudServiceConfiguration pools is batch mode.
                        type: str
                        choices:
                            - Batch
                            - Interactive
    metadata:
        description:
            - The Batch service does not assign any meaning to this metadata; it is solely for the use of user code.
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name of the metadata item.
                type: str
                required: true
            value:
                description:
                    - The value of the metadata item.
                type: str
                required: true
    start_task:
        description:
            - In an PATCH (update) operation, this property can be set to an empty object to remove the start task from the pool.
        type: dict
        suboptions:
            command_line:
                description:
                    - The command line does not run under a shell, and therefore cannot take advantage of shell features such as environment variable expansion.
                    - If you want to take advantage of such features, you should invoke the shell in the command line.
                    - Required if any other properties of the startTask are specified.
                type: str
            resource_files:
                description:
                    - A list of files that the Batch service will download to the compute node before running the command line.
                    - A single file or multiple files to be downloaded to a compute node.
                type: list
                elements: dict
                suboptions:
                    auto_storage_container_name:
                        description:
                            - The autoStorageContainerName, storageContainerUrl and httpUrl properties are mutually exclusive and one of them must be specified.
                        type: str
                    storage_container_url:
                        description:
                            - The autoStorageContainerName, storageContainerUrl and httpUrl properties are mutually exclusive and one of them must be specified.
                            - This URL must be readable and listable from compute nodes.
                            - There are three ways to get such a URL for a container in Azure storage.
                            - "Include a Shared Access Signature (SAS) granting read and list permissions on  the container,
                              use a managed identity with read and list permissions, or set the ACL for the container to allow public access."
                        type: str
                    http_url:
                        description:
                            - The autoStorageContainerName, storageContainerUrl and httpUrl properties are mutually exclusive and one of them must be specified.
                            - If the URL points to Azure Blob Storage, it must be readable from compute nodes.
                            - There are three ways to get such a URL for a blob in Azure storage.
                            - "Include a Shared Access Signature (SAS) granting read permissions on the blob, use a managed identity with read permission,
                              or set the ACL for the blob or its container to allow public access."
                        type: str
                    blob_prefix:
                        description:
                            - The property is valid only when autoStorageContainerName or storageContainerUrl is used.
                            - This prefix can be a partial filename or a subdirectory.
                            - If a prefix is not specified, all the files in the container will be downloaded.
                        type: str
                    file_path:
                        description:
                            - If the httpUrl property is specified, the filePath is required and
                              describes the path which the file will be downloaded to, including the filename.
                            - If the autoStorageContainerName or storageContainerUrl property is specified, filePath is optional and
                              is the directory to download the files to.
                            - "In the case where filePath is used as a directory, any directory structure already associated with the input data
                              will be retained in full and appended to the specified filePath directory."
                            - The specified relative path cannot break out of  the task's working directory.
                        type: str
                    file_mode:
                        description:
                            - This property applies only to files being downloaded to Linux compute nodes.
                            - It will be ignored if it is specified for a resourceFile which will be downloaded to a Windows node.
                            - If this property is not specified for a Linux node, then a default value of 0770 is applied to the file.
                        type: str
                    identity_reference:
                        description:
                            - The reference to a user assigned identity associated with the Batch pool which a compute node will use.
                        type: dict
                        suboptions:
                            resource_id:
                                description:
                                    - The ARM resource id of the user assigned identity.
                                type: str
            environment_settings:
                description:
                    - A list of environment variable settings for the start task.
                type: list
                elements: dict
                suboptions:
                    name:
                        description:
                            - The name of the environment variable.
                        type: str
                        required: true
                    value:
                        description:
                            - The value of the environment variable.
                        type: str
            user_identity:
                description:
                    - If omitted, the task runs as a non-administrative user unique to the task.
                type: dict
                suboptions:
                    user_name:
                        description:
                            - The username of the task user identity.
                            - The userName and autoUser properties are mutually exclusive; you must specify one but not both.
                        type: str
                    auto_user:
                        description:
                            - The autouser config of the task user identity.
                            - The userName and autoUser properties are mutually exclusive; you must specify one but not both.
                        type: dict
                        suboptions:
                            scope:
                                description:
                                    - The scope of the auto user scope.
                                    - The default value is Pool.
                                    - If the pool is running Windows a value of Task should be specified if stricter isolation between tasks is required.
                                type: str
                                default: Pool
                                choices:
                                    - Task
                                    - Pool
                            elevation_level:
                                description:
                                    - The auto user elevation level.
                                type: str
                                default: NonAdmin
                                choices:
                                    - NonAdmin
                                    - Admin
            max_task_retry_count:
                description:
                    - The Batch service retries a task if its exit code is nonzero.
                    - Note that this value specifically controls the number of retries.
                    - The Batch service will try the task once, and may then retry up to this limit.
                type: int
                default: 0
            wait_for_success:
                description:
                    - If true and the start task fails on a compute node.
                    - The Batch service retries the start task up to its maximum retry count (maxTaskRetryCount).
                    - "If the task has still not completed successfully after all retries, then the Batch service marks the compute node unusable,
                      and will not schedule tasks to it."
                    - This condition can be detected via the node state and scheduling error detail.
                    - If false, the Batch service will not wait for the start task to complet.
                type: bool
            container_settings:
                description:
                    - "When this is specified, all directories recursively below the AZ_BATCH_NODE_ROOT_DIR are mapped into the container,
                      all task environment variables are mapped into the container, and the task command line is executed in the container."
                type: dict
                suboptions:
                    container_run_options:
                        description:
                            - "These additional options are supplied as arguments to the 'docker create' command,
                              in addition to those controlled by the Batch Service."
                        type: str
                    image_name:
                        description:
                            - This is the full image reference, as would be specified to "docker pull".
                            - If no tag is provided as part of the image name, the tag ":latest" is used as a default.
                        type: str
                    registry:
                        description:
                            - This setting can be omitted if was already provided at pool creation.
                        type: dict
                        suboptions:
                            user_name:
                                description:
                                    - he user name to log into the registry server.
                                type: str
                            password:
                                description:
                                    - The password to log into the registry server.
                                type: str
                            registry_server:
                                description:
                                    - If omitted, the default is "docker.io".
                                type: str
                                default: 'docker.io'
                            identity_reference:
                                description:
                                    - The reference to a user assigned identity associated with the Batch pool which a compute node will use.
                                type: dict
                                suboptions:
                                    resource_id:
                                        description:
                                            - The ARM resource id of the user assigned identity.
                                        type: str
                    working_directory:
                        description:
                            - A flag to indicate where the container task working directory is.
                        type: str
                        choices:
                            - TaskWorkingDirectory
                            - ContainerImageDefault
    application_packages:
        description:
            - Changes to application package references affect all new compute nodes joining the pool.
            - But do not affect compute nodes that are already in the pool until they are rebooted or reimaged.
            - There is a maximum of 10 application package references on any given pool.
        type: list
        elements: dict
        suboptions:
            id:
                description:
                    - The ID of the application package to install.
                    - This must be inside the same batch account as the pool.
                    - This can either be a reference to a specific version or the default version if one exists.
                type: str
                required: true
            version:
                description:
                    - "If this is omitted, and no default version is specified for this application,
                      the request fails with the error code InvalidApplicationPackageReferences."
                    - If you are calling the REST API directly, the HTTP status code is 409.
                type: str
    certificates:
        description:
            - For Windows compute nodes, the Batch service installs the certificates to the specified certificate store and location.
            - "For Linux compute nodes, the certificates are stored in a directory inside the task working directory and
              an environment variable AZ_BATCH_CERTIFICATES_DIR is supplied to the task to query for this location."
            - "For certificates with visibility of 'remoteUser', a 'certs' directory is created in the user's home directory and
              certificates are placed in that directory."
        type: list
        elements: dict
        suboptions:
            id:
                description:
                    - The fully qualified ID of the certificate to install on the pool.
                    - This must be inside the same batch account as the pool.
                type: str
                required: true
            store_location:
                description:
                    - The default value is C(CurrentUser).
                    - This property is applicable only for pools configured with Windows nodes.
                    - "For Linux compute nodes, the certificates are stored in a directory inside the task working directory and
                      an environment variable AZ_BATCH_CERTIFICATES_DIR is supplied to the task to query for this location."
                    - "For certificates with visibility of 'remoteUser', a 'certs' directory is created in the user's home directory
                      and certificates are placed in that directory."
                type: str
                default: CurrentUser
                choices:
                    - CurrentUser
                    - LocalMachine
            store_name:
                description:
                    - This property is applicable only for pools configured with Windows nodes.
                    - This created with cloudServiceConfiguration, or with virtualMachineConfiguration using a Windows image reference.
                type: str
                default: My
                choices:
                    - My
                    - Root
                    - CA
                    - Trust
                    - Disallowed
                    - TrustedPeople
                    - TrustedPublisher
                    - AuthRoot
                    - AddressBook
            visibility:
                description:
                    - Which user accounts on the compute node should have access to the private data of the certificate.
                type: list
                elements: str
                choices:
                    - StartTask
                    - Task
                    - RemoteUser
    application_licenses:
        description:
            - The list of application licenses must be a subset of available Batch service application licenses.
            - If a license is requested which is not supported, pool creation will fail.
        type: list
        elements: str
    mount_configuration:
        description:
            - This supports Azure Files, NFS, CIFS/SMB, and Blobfuse.
        type: list
        elements: dict
        suboptions:
            azure_blob_file_system_configuration:
                description:
                    - Configuration of the azure blob file system.
                    - This property is mutually exclusive with all other properties.
                type: dict
                suboptions:
                    account_name:
                        description:
                            - The Azure Storage Account name.
                        type: str
                        required: true
                    container_name:
                        description:
                            - The Azure Blob Storage Container name.
                        type: str
                        required: true
                    account_key:
                        description:
                            - This property is mutually exclusive with both sasKey and identity; exactly one must be specified.
                        type: str
                    sas_key:
                        description:
                            - This property is mutually exclusive with both accountKey and identity; exactly one must be specified.
                        type: str
                    blobfuse_options:
                        description:
                            - These are C(net use) options in Windows and C(mount) options in Linux.
                        type: str
                        choices:
                            - 'net use'
                            - mount
                    relative_mount_path:
                        description:
                            - "All file systems are mounted relative to the Batch mounts directory,
                              accessible via the AZ_BATCH_NODE_MOUNTS_DIR environment variable."
                        type: str
                        required: true
                    identity_reference:
                        description:
                            - This property is mutually exclusive with both accountKey and sasKey; exactly one must be specified.
                        type: dict
                        suboptions:
                            resource_id:
                                description:
                                    - The ARM resource id of the user assigned identity.
                                type: str
            nfs_mount_configuration:
                description:
                    - Configuration of the nfs mount.
                    - This property is mutually exclusive with all other properties.
                type: dict
                suboptions:
                    source:
                        description:
                            - The URI of the file system to mount.
                        type: str
                        required: true
                    relative_mount_path:
                        description:
                            - "All file systems are mounted relative to the Batch mounts directory,
                              accessible via the AZ_BATCH_NODE_MOUNTS_DIR environment variable."
                        type: str
                        required: true
                    mount_options:
                        description:
                            - These are C(net use) options in Windows and C(mount) options in Linux.
                        type: str
                        choices:
                            - 'net use'
                            - mount
            cifs_mount_configuration:
                description:
                    - Configuration of the cifs mount.
                    - This property is mutually exclusive with all other properties.
                type: dict
                suboptions:
                    user_name:
                        description:
                            - The user to use for authentication against the CIFS file system.
                        type: str
                        required: true
                    source:
                        description:
                            - The URI of the file system to mount.
                        type: str
                        required: true
                    relative_mount_path:
                        description:
                            - "All file systems are mounted relative to the Batch mounts directory,
                              accessible via the AZ_BATCH_NODE_MOUNTS_DIR environment variable."
                        type: str
                        required: true
                    mount_options:
                        description:
                            -  These are C(net use) options in Windows and C(mount) options in Linux.
                        type: str
                        choices:
                            - 'net use'
                            - mount
                    password:
                        description:
                            - The password to use for authentication against the CIFS file system.
                        type: str
                        required: true
            azure_file_share_configuration:
                description:
                    - Configuration of the azure file share.
                    - This property is mutually exclusive with all other properties.
                type: dict
                suboptions:
                    account_name:
                        description:
                            - The Azure Storage account name.
                        type: str
                        required: true
                    azure_file_url:
                        description:
                            -  This is of the form 'https://{account}.file.core.windows.net/'.
                        type: str
                        required: true
                    account_key:
                        description:
                            - The Azure Storage account key.
                        type: str
                        required: true
                    relative_mount_path:
                        description:
                            - "All file systems are mounted relative to the Batch mounts directory,
                              accessible via the AZ_BATCH_NODE_MOUNTS_DIR environment variable."
                        type: str
                        required: true
                    mount_options:
                        description:
                            - These are C(net use) options in Windows and C(mount) options in Linux.
                        type: str
                        choices:
                            - 'net use'
                            - mount
    target_node_communication_mode:
        description:
            - If omitted, the default value is C(Default).
        type: str
        default: Default
        choices:
            - Default
            - Classic
            - Simplified
    upgrade_policy:
        description:
            - Describes an upgrade policy.
        type: dict
        suboptions:
            mode:
                description:
                    - Specifies the mode of an upgrade to virtual machines in the scale set.
                type: str
                choices:
                    - automatic
                    - manual
                    - rolling
                required: true
            automatic_os_upgrade_policy:
                description:
                    -  The configuration parameters used for performing automatic OS upgrade.
                type: dict
                suboptions:
                    disable_automatic_rollback:
                        description:
                            - Whether OS image rollback feature should be disabled.
                        type: bool
                    enable_automatic_os_upgrade:
                        description:
                            - "Indicates whether OS upgrades should automatically be applied to scale set instances in a rolling fashion
                              when a newer version of the OS image becomes available."
                        type: bool
                    use_rolling_upgrade_policy:
                        description:
                            - Indicates whether rolling upgrade policy should be used during Auto OS Upgrade.
                            - Auto OS Upgrade will fallback to the default policy if no policy is defined on the VMSS.
                        type: bool
                    os_rolling_upgrade_deferral:
                        description:
                            - Defer OS upgrades on the TVMs if they are running tasks.
                        type: bool
            rolling_upgrade_policy:
                description:
                    - This property is only supported on Pools with the virtualMachineConfiguration property.
                type: dict
                suboptions:
                    enable_cross_zone_upgrade:
                        description:
                            - Allow VMSS to ignore AZ boundaries when constructing upgrade batches.
                            - Take into consideration the Update Domain and maxBatchInstancePercent to determine the batch size.
                            - If this field is not set, Azure Azure Batch will not set its default value.
                            - "The value of enableCrossZoneUpgrade on the created VirtualMachineScaleSet
                              will be decided by the default configurations on VirtualMachineScaleSet."
                            - This field is able to be set to true or false only when using NodePlacementConfiguration as Zonal.
                        type: bool
                    max_batch_instance_percent:
                        description:
                            - The maximum percent of total virtual machine instances that will be upgraded simultaneously by the rolling upgrade in one batch.
                            - "As this is a maximum, unhealthy instances in previous or future batches can cause the percentage of instances
                              in a batch to decrease to ensure higher reliability."
                            - The value of this field should be between 5 and 100, inclusive.
                            - "If both maxBatchInstancePercent and maxUnhealthyInstancePercent are assigned with value,
                              the value of maxBatchInstancePercent should not be more than maxUnhealthyInstancePercent."
                        type: int
                    max_unhealthy_instance_percent:
                        description:
                            - "The maximum percentage of the total virtual machine instances in the scale set that can be simultaneously unhealthy,
                              either as a result of being upgraded, or by being found in an unhealthy state by the virtual machine health checks
                              before the rolling upgrade aborts."
                            - This constraint will be checked prior to starting any batch.
                            - "If both maxBatchInstancePercent and maxUnhealthyInstancePercent are assigned with value,
                              the value of maxBatchInstancePercent should not be more than maxUnhealthyInstancePercent."
                        type: int
                    max_unhealthy_upgraded_instance_percent:
                        description:
                            - The maximum percentage of upgraded virtual machine instances that can be found to be in an unhealthy state.
                            - This check will happen after each batch is upgraded.
                            - If this percentage is ever exceeded, the rolling update aborts.
                            - The value of this field should be between 0 and 100, inclusive.
                        type: int
                    pause_time_between_batches:
                        description:
                            - The wait time between completing the update for all virtual machines in one batch and starting the next batch.
                            - The time duration should be specified in ISO 8601 format.
                        type: str
                    prioritize_unhealthy_instances:
                        description:
                            - Upgrade all unhealthy instances in a scale set before any healthy instances.
                        type: bool
                    rollback_failed_instances_on_policy_breach:
                        description:
                            - Rollback failed instances to previous model if the Rolling Upgrade policy is violated.
                        type: bool
    resource_tags:
        description:
            - The user-defined tags to be associated with the Azure Batch Pool.
            - When specified, these tags are propagated to the backing Azure resources associated with the pool.
            - This property can only be specified when the Batch account was created with the poolAllocationMode property set to 'UserSubscription'.
        type: dict
    is_disable_auto_scale:
        description:
            - Whether disables automatic scaling for a pool.
        type: bool
        default: false
    is_stop_resize:
        description:
            - Whether stops an ongoing resize operation on the pool.
        type: bool
        default: false
    state:
        description:
            - Assert the state of the Batch Account Pool.
            - Use C(present) to create or update a Batch Account Pool and C(absent) to delete it.
        default: present
        type: str
        choices:
            - present
            - absent

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred Sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Create a new batch account pool
  azure_rm_batchaccountpool:
    resource_group: "{{ resource_group }}"
    batch_account_name: "{{ batch_account_name }}"
    name: "pool{{ batch_account_name }}--004"
    deployment_configuration:
      virtual_machine_configuration:
        image_reference:
          offer: ubuntu-hpc
          publisher: microsoft-dsvm
          sku: 2204
          version: latest
        node_agent_sku_id: batch.node.ubuntu 22.04
        node_placement_configuration:
          policy: Regional
        os_disk:
          caching: None
          managed_disk:
            storage_account_type: Premium_LRS
    display_name: "fredtest01"
    identity:
      type: UserAssigned
      user_assigned_identities:
        '/subscriptions/xxx-xxx/resourceGroups/testRG/providers/Microsoft.ManagedIdentity/userAssignedIdentities/AzSecPackAutoConfigUA-westus':
          client_id: xxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
          principal_id: yyyyyy-yyyy-yyyy-yyyy-yyyyyyyyyyyy
        '/subscriptions/xxx-xxx/resourceGroups/yishitest/providers/Microsoft.ManagedIdentity/userAssignedIdentities/ystestidentity':
          client_id: yyyyyy-yyyy-yyyy-yyyy-yyyyyyyyyyyy
          principal_id: xxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    inter_node_communication: Disabled
    network_configuration:
      dynamic_vnet_assignment_scope: none
      subnet_id: "/subscriptions/xxx-xxx/resourceGroups/testRG/providers/Microsoft.Network/virtualNetworks/vnet02/subnets/default"
      endpoint_configuration:
        inbound_nat_pools:
          - backend_port: 33
            frontend_port_range_end: 49999
            frontend_port_range_start: 1
            name: nat02
            protocol: UDP
      public_ip_address_configuration:
        provision: BatchManaged
    scale_settings:
      fixed_scale:
        resize_timeout: PT15M
        target_dedicated_nodes: 0
        target_low_priority_nodes: 0
    target_node_communication_mode: Default
    task_scheduling_policy:
      node_fill_type: Pack
    task_slots_per_node: 1
    vm_size: STANDARD_D2S_V3
    upgrade_policy:
      mode: manual
      rolling_upgrade_policy:
        max_batch_instance_percent: 20
        max_unhealthy_instance_percent: 20
        max_unhealthy_upgraded_instance_percent: 20
        pause_time_between_batches: P0D
        rollback_failed_instances_on_policy_breach: false
      automatic_os_upgrade_policy:
        disable_automatic_rollback: false
        enable_automatic_os_upgrade: false
        os_rolling_upgrade_deferral: false
        use_rolling_upgrade_policy: false

- name: Delete the Batch Account Pool
  azure_rm_batchaccountpool:
    resource_group: MyResGroup
    name: pool01
    batch_account_name: mybatchaccount
    state: absent
'''

RETURN = '''
state:
    description:
        - Contains information about an pool in a Batch account.
    type: dict
    returned: always
    sample: {
                "allocation_state": "Steady",
                "allocation_state_transition_time": "2024-11-05T08:58:16.803138Z",
                "batch_account_name": "fredbatch02",
                "creation_time": "2024-11-05T08:58:15.399345Z",
                "current_dedicated_nodes": 0,
                "current_low_priority_nodes": 0,
                "deployment_configuration": {
                    "virtual_machine_configuration": {
                        "image_reference": {
                            "offer": "ubuntu-hpc",
                            "publisher": "microsoft-dsvm",
                            "sku": "2204",
                            "version": "latest"
                        },
                        "node_agent_sku_id": "batch.node.ubuntu 22.04",
                        "node_placement_configuration": {
                            "policy": "Regional"
                        },
                        "os_disk": {
                            "caching": "None",
                            "managed_disk": {
                                "storage_account_type": "Premium_LRS"
                            }
                        }
                    }
                },
                "etag": "0x8DCFD77FC345CFE",
                "id": "/subscriptions/xxx-xxx/resourceGroups/testRG/providers/Microsoft.Batch/batchAccounts/batch01/pools/pool01",
                "inter_node_communication": "Disabled",
                "last_modified": "2024-11-05T08:58:15.399347Z",
                "name": "poolfredbatch02--002",
                "network_configuration": {
                    "dynamic_vnet_assignment_scope": "None",
                    "enable_accelerated_networking": false,
                    "endpoint_configuration": {
                        "inbound_nat_pools": [
                            {
                                "backend_port": 33,
                                "frontend_port_range_end": 49999,
                                "frontend_port_range_start": 1,
                                "name": "nat02",
                                "protocol": "UDP"
                            }
                        ]
                    },
                    "public_ip_address_configuration": {
                        "provision": "BatchManaged"
                    },
                    "subnet_id": "/subscriptions/xxx-xxx/resourceGroups/testrg/providers/Microsoft.Network/virtualNetworks/vnet01/subnets/default"
                },
                "provisioning_state": "Succeeded",
                "provisioning_state_transition_time": "2024-11-05T08:58:15.399345Z",
                "resize_operation_status": {
                    "node_deallocation_option": "Requeue",
                    "resize_timeout": "PT15M",
                    "start_time": "2024-11-05T08:58:15.399317Z",
                    "target_dedicated_nodes": 0
                },
                "resource_group": "v-xisuRG06",
                "scale_settings": {
                    "fixed_scale": {
                        "resize_timeout": "PT15M",
                        "target_dedicated_nodes": 0,
                        "target_low_priority_nodes": 0
                    }
                },
                "target_node_communication_mode": "Default",
                "task_scheduling_policy": {
                    "node_fill_type": "Pack"
                },
                "task_slots_per_node": 1,
                "type": "Microsoft.Batch/batchAccounts/pools",
                "upgrade_policy": {
                    "automatic_os_upgrade_policy": {
                        "disable_automatic_rollback": false,
                        "enable_automatic_os_upgrade": false,
                        "os_rolling_upgrade_deferral": false,
                        "use_rolling_upgrade_policy": false
                    },
                    "mode": "Manual",
                    "rolling_upgrade_policy": {
                        "max_batch_instance_percent": 20,
                        "max_unhealthy_instance_percent": 20,
                        "max_unhealthy_upgraded_instance_percent": 20,
                        "pause_time_between_batches": "P0D",
                        "rollback_failed_instances_on_policy_breach": false
                    }
                },
                "vm_size": "STANDARD_D2S_V3"
            }
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt

try:
    from azure.core.polling import LROPoller
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMBatchAccountPool(AzureRMModuleBaseExt):
    """Configuration class for an Azure RM Batch Account Pool resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                required=True,
                type='str'
            ),
            batch_account_name=dict(
                type='str',
                required=True,
            ),
            name=dict(
                required=True,
                type='str'
            ),
            identity=dict(
                type='dict',
                options=dict(
                    type=dict(type='str', required=True, choices=['None', 'UserAssigned']),
                    user_assigned_identities=dict(
                        type='dict',
                    )
                )
            ),
            display_name=dict(
                type='str'
            ),
            vm_size=dict(
                type='str'
            ),
            deployment_configuration=dict(
                type='dict',
                options=dict(
                    cloud_service_configuration=dict(
                        type='dict',
                        options=dict(
                            os_family=dict(type='str', default='*'),
                            os_version=dict(type='str')
                        )
                    ),
                    virtual_machine_configuration=dict(
                        type='dict',
                        options=dict(
                            image_reference=dict(
                                type='dict',
                                required=True,
                                options=dict(
                                    publisher=dict(type='str'),
                                    offer=dict(type='str'),
                                    sku=dict(type='str'),
                                    version=dict(type='str'),
                                    id=dict(type='str'),
                                )
                            ),
                            node_agent_sku_id=dict(type='str', required=True),
                            windows_configuration=dict(
                                type='dict',
                                options=dict(
                                    enable_automatic_updates=dict(type='bool', default=True)
                                )
                            ),
                            data_disks=dict(
                                type='list',
                                elements='dict',
                                options=dict(
                                    lun=dict(type='int', required=True),
                                    caching=dict(type='str', choices=['None', 'ReadOnly', 'ReadWrite']),
                                    disk_size_gb=dict(type='int', required=True),
                                    storage_account_type=dict(
                                        type='str',
                                        default='Standard_LRS',
                                        choices=['Standard_LRS', 'Premium_LRS', 'StandardSSD_LRS']
                                    )
                                )
                            ),
                            license_type=dict(
                                type='str'
                            ),
                            container_configuration=dict(
                                type='dict',
                                options=dict(
                                    type=dict(type='str', choices=['DockerCompatible', 'CriCompatible'], required=True),
                                    container_image_names=dict(type='list', elements='str'),
                                    container_registries=dict(
                                        type='list',
                                        elements='dict',
                                        options=dict(
                                            user_name=dict(type='str'),
                                            password=dict(type='str', no_log=True),
                                            registry_server=dict(type='str', default='docker.io'),
                                            identity_reference=dict(
                                                type='dict',
                                                options=dict(
                                                    resource_id=dict(type='str')
                                                )
                                            )
                                        )
                                    )
                                )
                            ),
                            disk_encryption_configuration=dict(
                                type='dict',
                                options=dict(
                                    targets=dict(type='list', elements='str', choices=['TemporaryDisk', 'OsDisk'])
                                )
                            ),
                            node_placement_configuration=dict(
                                type='dict',
                                options=dict(
                                    policy=dict(
                                        type='str',
                                        choices=['Zonal', 'Regional']
                                    )
                                ),
                            ),
                            extensions=dict(
                                type='list',
                                elements='dict',
                                options=dict(
                                    name=dict(type='str', required=True),
                                    publisher=dict(type='str', required=True),
                                    type=dict(type='str', required=True),
                                    type_handler_version=dict(type='str'),
                                    auto_upgrade_minor_version=dict(type='bool'),
                                    enable_automatic_upgrade=dict(type='bool'),
                                    provision_after_extensions=dict(type='list', elements='str'),
                                    settings=dict(type='json'),
                                    protected_settings=dict(type='json')
                                )
                            ),
                            os_disk=dict(
                                type='dict',
                                options=dict(
                                    caching=dict(
                                        type='str',
                                        choices=['ReadOnly', 'ReadWrite', 'None']
                                    ),
                                    write_accelerator_enabled=dict(type='bool'),
                                    disk_size_gb=dict(type='int'),
                                    managed_disk=dict(
                                        type='dict',
                                        options=dict(
                                            storage_account_type=dict(type='str', choices=["Standard_LRS", "Premium_LRS", "StandardSSD_LRS"])
                                        )
                                    ),
                                    ephemeral_os_disk_settings=dict(
                                        type='dict',
                                        options=dict(
                                            placement=dict(type='str')
                                        )
                                    )
                                )
                            ),
                            security_profile=dict(
                                type='dict',
                                options=dict(
                                    security_type=dict(type='str', default='trustedLaunch'),
                                    encryption_at_host=dict(type='bool'),
                                    uefi_settings=dict(
                                        type='dict',
                                        options=dict(
                                            secure_boot_enabled=dict(type='bool'),
                                            v_tpm_enabled=dict(type='bool')
                                        )
                                    )
                                )
                            ),
                            service_artifact_reference=dict(
                                type='dict',
                                options=dict(
                                    id=dict(type='str')
                                )
                            )
                        )
                    )
                )
            ),
            scale_settings=dict(
                type='dict',
                options=dict(
                    fixed_scale=dict(
                        type='dict',
                        options=dict(
                            resize_timeout=dict(type='str', default='P15M'),
                            target_dedicated_nodes=dict(type='int'),
                            target_low_priority_nodes=dict(type='int'),
                            node_deallocation_option=dict(type='str', choices=["Requeue", "Terminate", "TaskCompletion", "RetainedData"])
                        )
                    ),
                    auto_scale=dict(
                        type='dict',
                        options=dict(
                            formula=dict(type='str', required=True),
                            evaluation_interval=dict(type='str', default='P15M'),
                        )
                    )
                )
            ),
            inter_node_communication=dict(
                type='str',
                choices=['Enabled', 'Disabled']
            ),
            network_configuration=dict(
                type='dict',
                options=dict(
                    subnet_id=dict(type='str'),
                    dynamic_vnet_assignment_scope=dict(type='str', choices=['none', 'job']),
                    endpoint_configuration=dict(
                        type='dict',
                        options=dict(
                            inbound_nat_pools=dict(
                                type='list',
                                elements='dict',
                                options=dict(
                                    name=dict(type='str', required=True),
                                    protocol=dict(type='str', choices=['UDP', 'TCP'], required=True),
                                    backend_port=dict(type='int', required=True),
                                    frontend_port_range_start=dict(type='int', required=True),
                                    frontend_port_range_end=dict(type='int', required=True),
                                    network_security_group_rules=dict(
                                        type='list',
                                        elements='dict',
                                        options=dict(
                                            priority=dict(type='int', required=True),
                                            access=dict(type='str', required=True, choices=['Allow', 'Deny']),
                                            source_address_prefix=dict(type='str'),
                                            source_port_ranges=dict(type='list', elements='str')
                                        )
                                    )
                                )
                            )
                        )
                    ),
                    public_ip_address_configuration=dict(
                        type='dict',
                        options=dict(
                            provision=dict(type='str', default='BatchManaged', choices=['BatchManaged', 'UserManaged', 'NoPublicIPAddresses']),
                            ip_address_ids=dict(
                                type='list',
                                elements='str'
                            )
                        )
                    ),
                    enable_accelerated_networking=dict(
                        type='bool'
                    )
                )
            ),
            task_slots_per_node=dict(
                type='int',
                default=1
            ),
            task_scheduling_policy=dict(
                type='dict',
                options=dict(
                    node_fill_type=dict(type='str', choices=['Spread', 'Pack'])
                )
            ),
            user_accounts=dict(
                type='list',
                elements='dict',
                options=dict(
                    name=dict(type='str', required=True),
                    password=dict(type='str', required=True, no_log=True),
                    elevation_level=dict(type='str', choices=['NonAdmin', 'Admin'], default='NonAdmin'),
                    linux_user_configuration=dict(
                        type='dict',
                        options=dict(
                            uid=dict(type='int',),
                            gid=dict(type='int'),
                            ssh_private_key=dict(type='str', no_log=True),
                        )
                    ),
                    windows_user_configuration=dict(
                        type='dict',
                        options=dict(
                            login_mode=dict(type='str', choices=['Batch', 'Interactive'])
                        )
                    )
                )
            ),
            metadata=dict(
                type='list',
                elements='dict',
                options=dict(
                    name=dict(type='str', required=True),
                    value=dict(type='str', required=True)
                )
            ),
            start_task=dict(
                type='dict',
                options=dict(
                    command_line=dict(type='str'),
                    resource_files=dict(
                        type='list',
                        elements='dict',
                        options=dict(
                            auto_storage_container_name=dict(type='str'),
                            storage_container_url=dict(type='str'),
                            http_url=dict(type='str'),
                            blob_prefix=dict(type='str'),
                            file_path=dict(type='str'),
                            file_mode=dict(type='str'),
                            identity_reference=dict(
                                type='dict',
                                options=dict(
                                    resource_id=dict(type='str')
                                )
                            )
                        )
                    ),
                    environment_settings=dict(
                        type='list',
                        elements='dict',
                        options=dict(
                            name=dict(type='str', required=True),
                            value=dict(type='str')
                        )
                    ),
                    user_identity=dict(
                        type='dict',
                        options=dict(
                            user_name=dict(type='str'),
                            auto_user=dict(
                                type='dict',
                                options=dict(
                                    scope=dict(type='str', default='Pool', choices=['Pool', 'Task']),
                                    elevation_level=dict(type='str', default='NonAdmin', choices=['NonAdmin', 'Admin'])
                                )
                            )
                        )
                    ),
                    max_task_retry_count=dict(
                        type='int',
                        default=0,
                    ),
                    wait_for_success=dict(type='bool'),
                    container_settings=dict(
                        type='dict',
                        options=dict(
                            container_run_options=dict(type='str'),
                            image_name=dict(type='str'),
                            working_directory=dict(type='str', choices=['TaskWorkingDirectory', 'ContainerImageDefault']),
                            registry=dict(
                                type='dict',
                                options=dict(
                                    user_name=dict(type='str'),
                                    password=dict(type='str', no_log=True),
                                    registry_server=dict(type='str', default='docker.io'),
                                    identity_reference=dict(
                                        type='dict',
                                        options=dict(
                                            resource_id=dict(type='str')
                                        )
                                    )
                                )
                            )
                        )
                    )
                )
            ),
            application_packages=dict(
                type='list',
                elements='dict',
                options=dict(
                    id=dict(type='str', required=True),
                    version=dict(type='str')
                )
            ),
            certificates=dict(
                type='list',
                elements='dict',
                options=dict(
                    id=dict(type='str', required=True),
                    store_location=dict(type='str', default='CurrentUser', choices=['CurrentUser', 'LocalMachine']),
                    store_name=dict(
                        type='str',
                        default='My',
                        choices=['My', 'Root', 'CA', 'Trust', 'Disallowed', 'TrustedPeople', 'TrustedPublisher', 'AuthRoot', 'AddressBook']
                    ),
                    visibility=dict(type='list', elements='str', choices=['StartTask', 'Task', 'RemoteUser'])
                )
            ),
            application_licenses=dict(
                type='list',
                elements='str'
            ),
            mount_configuration=dict(
                type='list',
                elements='dict',
                options=dict(
                    azure_blob_file_system_configuration=dict(
                        type='dict',
                        options=dict(
                            account_name=dict(type='str', required=True),
                            container_name=dict(type='str', required=True),
                            account_key=dict(type='str', no_log=True),
                            sas_key=dict(type='str', no_log=True),
                            blobfuse_options=dict(type='str', choices=['net use', 'mount']),
                            relative_mount_path=dict(type='str', required=True),
                            identity_reference=dict(
                                type='dict',
                                options=dict(
                                    resource_id=dict(type='str')
                                )
                            )
                        )
                    ),
                    nfs_mount_configuration=dict(
                        type='dict',
                        options=dict(
                            source=dict(type='str', required=True),
                            relative_mount_path=dict(type='str', required=True),
                            mount_options=dict(type='str', choices=['net use', 'mount'])
                        )
                    ),
                    cifs_mount_configuration=dict(
                        type='dict',
                        options=dict(
                            user_name=dict(type='str', required=True),
                            source=dict(type='str', required=True),
                            relative_mount_path=dict(type='str', required=True),
                            mount_options=dict(type='str', choices=['net use', 'mount']),
                            password=dict(type='str', no_log=True, required=True)
                        )
                    ),
                    azure_file_share_configuration=dict(
                        type='dict',
                        options=dict(
                            account_name=dict(type='str', required=True),
                            azure_file_url=dict(type='str', required=True),
                            account_key=dict(type='str', required=True, no_log=True),
                            relative_mount_path=dict(type='str', required=True),
                            mount_options=dict(type='str', choices=['net use', 'mount'])
                        )
                    )
                )
            ),
            target_node_communication_mode=dict(
                type='str',
                default='Default',
                choices=["Default", "Classic", "Simplified"]
            ),
            upgrade_policy=dict(
                type='dict',
                options=dict(
                    mode=dict(type='str', required=True, choices=['automatic', 'manual', 'rolling']),
                    automatic_os_upgrade_policy=dict(
                        type='dict',
                        options=dict(
                            disable_automatic_rollback=dict(type='bool'),
                            enable_automatic_os_upgrade=dict(type='bool'),
                            use_rolling_upgrade_policy=dict(type='bool'),
                            os_rolling_upgrade_deferral=dict(type='bool'),
                        )
                    ),
                    rolling_upgrade_policy=dict(
                        type='dict',
                        options=dict(
                            enable_cross_zone_upgrade=dict(type='bool'),
                            max_batch_instance_percent=dict(type='int'),
                            max_unhealthy_instance_percent=dict(type='int'),
                            max_unhealthy_upgraded_instance_percent=dict(type='int'),
                            pause_time_between_batches=dict(type='str'),
                            prioritize_unhealthy_instances=dict(type='bool'),
                            rollback_failed_instances_on_policy_breach=dict(type='bool')
                        )
                    )
                )
            ),
            resource_tags=dict(
                type='dict',
            ),
            is_disable_auto_scale=dict(
                type='bool',
                default=False
            ),
            is_stop_resize=dict(
                type='bool',
                default=False
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.batch_account_name = None
        self.name = None
        self.resource_group = None
        self.is_disable_auto_scale = None
        self.is_stop_resize = None
        self.results = dict(changed=False)
        self.state = None
        self.body = dict()

        super(AzureRMBatchAccountPool, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                      supports_check_mode=True,
                                                      supports_tags=False)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()):
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.body[key] = kwargs[key]

        response = None
        changed = False

        old_response = self.get_batchaccount_pool()

        if not old_response:
            self.log("Batch Account Pool instance doesn't exist")
            if self.state == 'absent':
                self.log("Old instance didn't exist")
            else:
                changed = True
                if not self.check_mode:
                    response = self.create_batchaccount_pool()
        else:
            self.log("Batch Account Pool instance already exists")
            if self.state == 'absent':
                if not self.check_mode:
                    changed = True
                    response = self.delete_batchaccount_pool()
            else:
                if not self.default_compare({}, self.body, old_response, '', dict(compare=[])):
                    changed = True
                if not self.check_mode and changed:
                    response = self.update_batchaccount_pool()
                if self.is_stop_resize and not self.check_mode:
                    changed = True
                    self.stop_resize_pool()
                if self.is_disable_auto_scale and not self.check_mode:
                    changed = True
                    self.disable_auto_scale_pool()

        self.results = dict(
            changed=changed,
            state=self.get_batchaccount_pool(),
        )
        return self.results

    def create_batchaccount_pool(self):
        '''
        Creates Batch Account Pool with the specified configuration.
        '''
        self.log("Creating the Batch Account Pool instance {0}".format(self.name))

        try:
            response = self.batch_account_client.pool.create(resource_group_name=self.resource_group,
                                                             account_name=self.batch_account_name,
                                                             pool_name=self.name,
                                                             parameters=self.batch_account_model.Pool(**self.body))
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as exc:
            self.log('Error attempting to create the Batch Account Pool instance.')
            self.fail("Error creating the Batch Account Pool instance: {0}".format(str(exc)))
        return response.as_dict()

    def update_batchaccount_pool(self):
        '''
        Update Batch Account Pool with the specified configuration.
        '''
        self.log("Updating the Batch Account Pool instance {0}".format(self.name))

        try:
            response = self.batch_account_client.pool.update(resource_group_name=self.resource_group,
                                                             account_name=self.batch_account_name,
                                                             pool_name=self.name,
                                                             parameters=self.batch_account_model.Pool(**self.body))
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as exc:
            self.log('Error attempting to update the Batch Account Pool instance.')
            self.fail("Error updating the Batch Account Pool instance: {0}".format(str(exc)))
        return response.as_dict()

    def disable_auto_scale_pool(self):
        '''
        Disable_auto_scale Batch Account Pool instance in the specified subscription and resource group.
        :return: True
        '''
        self.log("Disable_auto_scale the Batch Account Pool instance {0}".format(self.name))
        try:
            response = self.batch_account_client.pool.disable_auto_scale(resource_group_name=self.resource_group,
                                                                         account_name=self.batch_account_name,
                                                                         pool_name=self.name)
        except Exception as e:
            self.log('Error attempting to disable the Batch Account Pool auto_scale.')
            self.fail("Error disable the Batch Account Pool auto_scale instance: {0}".format(str(e)))
        return True

    def stop_resize_pool(self):
        '''
        Stop resize Batch Account Pool instance in the specified subscription and resource group.
        :return: True
        '''
        self.log("Stop the Batch Account Pool resize {0}".format(self.name))
        try:
            response = self.batch_account_client.pool.stop_resize(resource_group_name=self.resource_group,
                                                                  account_name=self.batch_account_name,
                                                                  pool_name=self.name)
        except Exception as e:
            self.log('Error attempting to stop the Batch Account Pool resize.')
            self.fail("Error stop the Batch Account Pool resize: {0}".format(str(e)))
        return True

    def delete_batchaccount_pool(self):
        '''
        Deletes specified Batch Account Pool instance in the specified subscription and resource group.
        :return: True
        '''
        self.log("Deleting the Batch Account Pool instance {0}".format(self.name))
        try:
            response = self.batch_account_client.pool.begin_delete(resource_group_name=self.resource_group,
                                                                   account_name=self.batch_account_name,
                                                                   pool_name=self.name)
        except Exception as e:
            self.log('Error attempting to delete the Batch Account Pool instance.')
            self.fail("Error deleting the Batch Account Pool instance: {0}".format(str(e)))

        return True

    def get_batchaccount_pool(self):
        '''
        Gets the properties of the specified Batch Account Pool
        :return: deserialized Batch Account Pool instance state dictionary
        '''
        self.log("Checking if the Batch Account Pool instance {0} is present".format(self.name))
        found = False
        try:
            response = self.batch_account_client.pool.get(resource_group_name=self.resource_group,
                                                          account_name=self.batch_account_name,
                                                          pool_name=self.name)
            found = True
            self.log("Response : {0}".format(response))
            self.log("Batch Account Pool instance : {0} found".format(response.name))
        except ResourceNotFoundError as e:
            self.log('Did not find the Batch Account Pool instance. Exception as {0}'.format(e))
        if found is True:
            return self.format_item(response.as_dict())
        return False

    def format_item(self, item):
        if item is None:
            return
        result = item
        result['resource_group'] = self.resource_group
        result['batch_account_name'] = self.batch_account_name
        return result


def main():
    """Main execution"""
    AzureRMBatchAccountPool()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_gpu_policy
short_description: GPU policy configuration for Cisco Intersight
description:
  - GPU policy configuration for Cisco Intersight.
  - Used to configure Multi-Instance GPU (MIG) partitioning and GPU assignment on server profiles.
  - Supports NVIDIA H100, B200, and other MIG-capable GPUs managed through Intersight.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles and Policies that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the GPU policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  description:
    description:
      - The user-defined description of the GPU policy.
    type: str
    aliases: [descr]
  gpu_mode:
    description:
      - The operating mode for GPUs in the server.
      - C(graphics) configures GPUs for standard graphics/display use.
      - C(compute) configures GPUs for compute workloads (CUDA, inference, training).
    type: str
    choices: [graphics, compute]
    default: compute
  mig_enabled:
    description:
      - Enable or disable Multi-Instance GPU (MIG) partitioning.
      - When enabled, each physical GPU can be partitioned into multiple isolated GPU instances.
      - MIG is supported on NVIDIA A100, H100, B200 and newer architectures.
    type: bool
    default: false
  mig_profiles:
    description:
      - List of MIG partition profiles to apply when mig_enabled is true.
      - Each entry defines the partition configuration for a GPU slot.
      - If fewer profiles are specified than available GPUs, remaining GPUs use the default profile.
    type: list
    elements: dict
    suboptions:
      slot_id:
        description:
          - The GPU slot number (1-8) this profile applies to.
        type: int
        required: true
      partition_profile:
        description:
          - The MIG partition profile name.
          - Common profiles for H100/B200 include 1g.10gb, 2g.20gb, 3g.40gb, 4g.40gb, 7g.80gb.
          - Use C(none) to disable MIG on this specific slot.
        type: str
        required: true
      instance_count:
        description:
          - Number of MIG instances to create with the specified partition profile.
          - Maximum depends on the GPU model and partition profile selected.
        type: int
        default: 1
  gpu_thermal_threshold:
    description:
      - GPU thermal throttle threshold temperature in Celsius.
      - When GPU temperature exceeds this threshold, performance will be throttled to prevent damage.
      - Valid range depends on GPU model. Typical values are 80-95 for data center GPUs.
      - Set to 0 to use the platform default.
    type: int
  power_capping_enabled:
    description:
      - Enable or disable GPU power capping.
      - When enabled, GPU power consumption is limited to the power_cap_watts value.
    type: bool
    default: false
  power_cap_watts:
    description:
      - Maximum power consumption per GPU in watts when power_capping_enabled is true.
      - Valid range depends on GPU model (e.g., H100 SXM supports 300-700W).
      - Set to 0 to use the platform default TDP.
    type: int
author:
  - Steve Fulmer (@stevefulme1)
'''

EXAMPLES = r'''
- name: Create GPU compute policy without MIG
  cisco.intersight.intersight_gpu_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: AI-Factory
    name: gpu-compute-default
    description: Standard GPU compute mode for inference
    gpu_mode: compute

- name: Create GPU policy with MIG partitioning for H100
  cisco.intersight.intersight_gpu_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: gpu-mig-inference
    description: MIG partitioned for multi-tenant inference
    gpu_mode: compute
    mig_enabled: true
    mig_profiles:
      - slot_id: 1
        partition_profile: 3g.40gb
        instance_count: 2
      - slot_id: 2
        partition_profile: 7g.80gb
        instance_count: 1

- name: Create GPU policy with thermal and power limits
  cisco.intersight.intersight_gpu_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: gpu-power-managed
    description: Power-managed GPU policy for dense deployments
    gpu_mode: compute
    gpu_thermal_threshold: 85
    power_capping_enabled: true
    power_cap_watts: 400

- name: Delete GPU policy
  cisco.intersight.intersight_gpu_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: gpu-compute-default
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "gpu-compute-default",
        "ObjectType": "graphics.GpuPolicy",
        "GpuMode": "compute",
        "MigEnabled": false
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    mig_profile = dict(
        slot_id=dict(type='int', required=True),
        partition_profile=dict(type='str', required=True),
        instance_count=dict(type='int', default=1),
    )
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        gpu_mode=dict(type='str', choices=['graphics', 'compute'], default='compute'),
        mig_enabled=dict(type='bool', default=False),
        mig_profiles=dict(type='list', elements='dict', options=mig_profile),
        gpu_thermal_threshold=dict(type='int'),
        power_capping_enabled=dict(type='bool', default=False),
        power_cap_watts=dict(type='int'),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    resource_path = '/graphics/GpuPolicies'

    intersight.api_body = {
        'Name': intersight.module.params['name'],
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
    }

    if module.params['state'] == 'present':
        intersight.set_tags_and_description()

        intersight.api_body['GpuMode'] = module.params['gpu_mode']
        intersight.api_body['MigEnabled'] = module.params['mig_enabled']

        # MIG partition profiles
        if module.params['mig_enabled'] and module.params.get('mig_profiles'):
            mig_configs = []
            for profile in module.params['mig_profiles']:
                mig_configs.append({
                    'SlotId': profile['slot_id'],
                    'PartitionProfile': profile['partition_profile'],
                    'InstanceCount': profile['instance_count'],
                })
            intersight.api_body['MigProfiles'] = mig_configs

        # Thermal threshold
        if module.params.get('gpu_thermal_threshold') is not None:
            intersight.api_body['GpuThermalThreshold'] = module.params['gpu_thermal_threshold']

        # Power capping
        if module.params['power_capping_enabled']:
            intersight.api_body['PowerCappingEnabled'] = True
            if module.params.get('power_cap_watts') is not None:
                intersight.api_body['PowerCapWatts'] = module.params['power_cap_watts']
        else:
            intersight.api_body['PowerCappingEnabled'] = False

    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_wireless_alternate_management_interface_ipv6
short_description: Resource module for devices _wireless _alternate _management _interface
  _ipv6
description:
  - Manage operation update of the resource devices _wireless _alternate _management
    _interface _ipv6.
  - Update alternate management interface IPv6 address.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  addresses:
    description: Configured alternate management interface addresses.
    elements: dict
    suboptions:
      address:
        description: The IP address configured for the alternate management interface.
        type: str
      assignmentMode:
        description: The type of address assignment. Either static or dynamic.
        type: str
      gateway:
        description: The gateway address configured for the alternate managment interface.
        type: str
      nameservers:
        description: The DNS servers settings for this address.
        suboptions:
          addresses:
            description: Up to 2 nameserver addresses to use, ordered in priority
              from highest to lowest priority.
            elements: str
            type: list
        type: dict
      prefix:
        description: The IPv6 prefix length of the IPv6 interface. Required if IPv6
          object is included.
        type: str
      protocol:
        description: The IP protocol used for the address.
        type: str
    type: list
  serial:
    description: Serial path parameter.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless updateDeviceWirelessAlternateManagementInterfaceIpv6
    description: Complete reference of the updateDeviceWirelessAlternateManagementInterfaceIpv6
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-device-wireless-alternate-management-interface-ipv6
notes:
  - SDK Method used are
    wireless.Wireless.update_device_wireless_alternate_management_interface_ipv6,
  - Paths used are
    put /devices/{serial}/wireless/alternateManagementInterface/ipv6,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.devices_wireless_alternate_management_interface_ipv6:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    addresses:
      - address: 2001:db8:3c4d:15::1
        assignmentMode: static
        gateway: fe80:db8:c15:c0:d0c::10ca:1d02
        nameservers:
          addresses:
            - 2001:db8:3c4d:15::1
            - 2001:db8:3c4d:15::1
        prefix: 2001:db8:3c4d:15::/64
        protocol: ipv6
    serial: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "addresses": [
        {
          "protocol": "string",
          "assignmentMode": "string",
          "address": "string",
          "gateway": "string",
          "prefix": "string",
          "nameservers": {
            "addresses": [
              "string"
            ]
          }
        }
      ]
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_set_anti_malware_update_schedule
short_description: Set both Anti-Bot and Anti-Virus update schedules.
description:
  - Set both Anti-Bot and Anti-Virus update schedules.
  - All operations are performed over Web Services API.
  - Available from R82 JHF management version.
version_added: "6.5.0"
author: "Dor Berenstein (@chkp-dorbe)"
options:
  enabled:
    description:
      - Enable/Disable Anti-Malware Update Schedule.
    type: bool
  schedule:
    description:
      - Schedule Configuration.
    type: dict
    suboptions:
      time:
        description:
          - Time in format HH,mm.
        type: str
      recurrence:
        description:
          - Days recurrence.
        type: dict
        suboptions:
          pattern:
            description:
              - Days recurrence pattern.
            type: str
            choices: ['Daily', 'Weekly', 'Monthly', 'Interval']
          interval_hours:
            description:
              - The amount of hours between updates. <font color="red">Required only when</font> pattern is set to 'Interval'.
            type: int
          interval_minutes:
            description:
              - The amount of minutes between updates. <font color="red">Required only when</font> pattern is set to 'Interval'.
            type: int
          interval_seconds:
            description:
              - The amount of seconds between updates. <font color="red">Required only when</font> pattern is set to 'Interval'.
            type: int
          weekdays:
            description:
              - Days of the week to run the update.<br> Valid values, group of values from {'Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'}.
                <font color="red">Required only when</font> pattern is set to 'Weekly'.
            type: list
            elements: str
          days:
            description:
              - Days of the month to run the update.<br> Valid values, interval in the range of 1 to 31. <font color="red">Required only
                when</font> pattern is set to 'Monthly'.
            type: list
            elements: str
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: set-anti-malware-update-schedule
  cp_mgmt_set_anti_malware_update_schedule:
    schedule:
      recurrence:
        interval_hours: 4
        interval_minutes: 30
        interval_seconds: 10
        pattern: interval
    state: present
"""

RETURN = """
cp_mgmt_set_anti_malware_update_schedule:
  description: The checkpoint set-anti-malware-update-schedule output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_commands, api_command


def main():
    argument_spec = dict(
        enabled=dict(type='bool'),
        schedule=dict(type='dict', options=dict(
            time=dict(type='str'),
            recurrence=dict(type='dict', options=dict(
                pattern=dict(type='str', choices=['Daily', 'Weekly', 'Monthly', 'Interval']),
                interval_hours=dict(type='int'),
                interval_minutes=dict(type='int'),
                interval_seconds=dict(type='int'),
                weekdays=dict(type='list', elements="str"),
                days=dict(type='list', elements="str")
            ))
        ))
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "set-anti-malware-update-schedule"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2016, Thomas Stringer <tomstr@microsoft.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_loadbalancer_info

version_added: "0.1.2"

short_description: Get load balancer facts

description:
    - Get facts for a specific load balancer or all load balancers.

options:
    name:
        description:
            - Limit results to a specific resource group.
        type: str
    resource_group:
        description:
            - The resource group to search for the desired load balancer.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Thomas Stringer (@trstringer)
'''

EXAMPLES = '''
- name: Get facts for one load balancer
  azure_rm_loadbalancer_info:
    name: Testing
    resource_group: myResourceGroup

- name: Get facts for all load balancers
  azure_rm_loadbalancer_info:

- name: Get facts for all load balancers in a specific resource group
  azure_rm_loadbalancer_info:
    resource_group: myResourceGroup

- name: Get facts by tags
  azure_rm_loadbalancer_info:
    tags:
      - testing
'''

RETURN = '''
loadbalancers:
    description:
        - Gets a list of load balancers.
    returned: always
    type: list
    elements: dict
    sample: [
        {
            "etag": "1c83ade9-9dee-4027-860a-d5fabacc184f",
            "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myAzureResourceGroup/providers/
                   Microsoft.Network/loadBalancers/testloadbalancer1",
            "location": "centralindia",
            "name": "testloadbalancer1",
            "properties": {
                "backendAddressPools": [],
                "frontendIPConfigurations": [
                    {
                        "etag": "1c83ade9-9dee-4027-860a-d5fabacc184f",
                        "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myAzureResourceGroup/
                               providers/Microsoft.Network/loadBalancers/testloadbalancer1/
                               frontendIPConfigurations/frontendipconf0",
                        "name": "frontendipconf0",
                        "properties": {
                            "privateIPAllocationMethod": "Dynamic",
                            "provisioningState": "Succeeded",
                            "publicIPAddress": {
                                "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/
                                       myAzureResourceGroup/providers/Microsoft.Network/publicIPAddresses/testpip"
                            }
                        },
                        "zones": ["1", "2", "3"],
                        "type": "Microsoft.Network/loadBalancers/frontendIPConfigurations"
                    }
                ],
                "inboundNatPools": [],
                "inboundNatRules": [],
                "loadBalancingRules": [],
                "outboundRules": [],
                "probes": [],
                "provisioningState": "Succeeded",
                "resourceGuid": "0b31ab3e-7c55-438d-92a0-5acdc99b5277"
            },
            "sku": {
                "name": "Standard"
            },
            "type": "Microsoft.Network/loadBalancers"
        }
    ]
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
except Exception:
    # handled in azure_rm_common
    pass

AZURE_OBJECT_CLASS = 'LoadBalancer'


class AzureRMLoadBalancerInfo(AzureRMModuleBase):
    """Utility class to get load balancer facts"""

    def __init__(self):

        self.module_args = dict(
            name=dict(type='str'),
            resource_group=dict(type='str'),
            tags=dict(type='list', elements='str')
        )

        self.results = dict(
            changed=False,
            loadbalancers=[]
        )

        self.name = None
        self.resource_group = None
        self.tags = None

        super(AzureRMLoadBalancerInfo, self).__init__(
            derived_arg_spec=self.module_args,
            supports_check_mode=True,
            supports_tags=False,
            facts_module=True
        )

    def exec_module(self, **kwargs):

        for key in self.module_args:
            setattr(self, key, kwargs[key])

        self.results['loadbalancers'] = (
            self.get_item() if self.name
            else self.list_items()
        )

        return self.results

    def get_item(self):
        """Get a single load balancer"""

        self.log('Get properties for {0}'.format(self.name))

        item = None
        result = []

        try:
            item = self.network_client.load_balancers.get(self.resource_group, self.name)
        except ResourceNotFoundError:
            pass

        if item and self.has_tags(item.tags, self.tags):
            result = [self.serialize_obj(item, AZURE_OBJECT_CLASS)]

        return result

    def list_items(self):
        """Get all load balancers"""

        self.log('List all load balancers')

        if self.resource_group:
            try:
                response = self.network_client.load_balancers.list(self.resource_group)
            except ResourceNotFoundError as exc:
                self.fail('Failed to list items in resource group {0} - {1}'.format(self.resource_group, str(exc)))
        else:
            try:
                response = self.network_client.load_balancers.list_all()
            except ResourceNotFoundError as exc:
                self.fail('Failed to list all items - {0}'.format(str(exc)))

        results = []
        for item in response:
            if self.has_tags(item.tags, self.tags):
                results.append(self.serialize_obj(item, AZURE_OBJECT_CLASS))

        return results


def main():
    """Main module execution code path"""

    AzureRMLoadBalancerInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_vpc_subnet_info
version_added: 1.0.0
short_description: Gather information about EC2 VPC subnets in AWS
description:
    - Gather information about EC2 VPC subnets in AWS.
author: "Rob White (@wimnat)"
options:
  subnet_ids:
    description:
      - A list of subnet IDs to gather information for.
    aliases: ['subnet_id']
    type: list
    elements: str
    default: []
  filters:
    description:
      - A dict of filters to apply. Each dict item consists of a filter key and a filter value.
        See U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeSubnets.html) for possible filters.
    type: dict
    default: {}
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

# Gather information about all VPC subnets
- amazon.aws.ec2_vpc_subnet_info:

# Gather information about a particular VPC subnet using ID
- amazon.aws.ec2_vpc_subnet_info:
    subnet_ids: subnet-00112233

# Gather information about any VPC subnet with a tag key Name and value Example
- amazon.aws.ec2_vpc_subnet_info:
    filters:
      "tag:Name": Example

# Gather information about any VPC subnet within VPC with ID vpc-abcdef00
- amazon.aws.ec2_vpc_subnet_info:
    filters:
      vpc-id: vpc-abcdef00

# Gather information about a set of VPC subnets, publicA, publicB and publicC within a
# VPC with ID vpc-abcdef00 and then use the jinja map function to return the
# subnet_ids as a list.

- amazon.aws.ec2_vpc_subnet_info:
    filters:
      vpc-id: vpc-abcdef00
      "tag:Name": "{{ item }}"
  loop:
    - publicA
    - publicB
    - publicC
  register: subnet_info

- set_fact:
    subnet_ids: "{{ subnet_info.results | sum(attribute='subnets', start=[]) | map(attribute='subnet_id') }}"
"""

RETURN = r"""
subnets:
    description: Returns an array of complex objects as described below.
    returned: success
    type: complex
    contains:
        id:
            description: Subnet resource id.
            returned: always
            type: str
            sample: subnet-b883b2c4
        cidr_block:
            description: The IPv4 CIDR of the Subnet.
            returned: always
            type: str
            sample: "10.0.0.0/16"
        ipv6_cidr_block:
            description: The IPv6 CIDR block actively associated with the Subnet.
            returned: always
            type: str
            sample: "2001:db8:0:102::/64"
        availability_zone:
            description: Availability zone of the Subnet.
            returned: always
            type: str
            sample: us-east-1a
        availability_zone_id:
            description: The AZ ID of the subnet.
            returned: always
            type: str
            sample: use1-az6
        state:
            description: state of the Subnet.
            returned: always
            type: str
            sample: available
        tags:
            description: tags attached to the Subnet, includes name.
            returned: always
            type: dict
            sample: {"Name": "My Subnet", "env": "staging"}
        map_public_ip_on_launch:
            description: whether public IP is auto-assigned to new instances.
            returned: always
            type: bool
            sample: false
        assign_ipv6_address_on_creation:
            description: whether IPv6 address is auto-assigned to new instances.
            returned: always
            type: bool
            sample: false
        vpc_id:
            description: the id of the VPC where this Subnet exists.
            returned: always
            type: str
            sample: vpc-67236184
        available_ip_address_count:
            description: number of available IPv4 addresses.
            returned: always
            type: str
            sample: 251
        default_for_az:
            description: indicates whether this is the default Subnet for this Availability Zone.
            returned: always
            type: bool
            sample: false
        enable_dns64:
            description:
            - Indicates whether DNS queries made should return synthetic IPv6 addresses for IPv4-only destinations.
            type: bool
            sample: false
        ipv6_association_id:
            description: The IPv6 association ID for the currently associated CIDR.
            returned: always
            type: str
            sample: subnet-cidr-assoc-b85c74d2
        ipv6_native:
            description: Indicates whether this is an IPv6 only subnet.
            type: bool
            sample: false
        ipv6_cidr_block_association_set:
            description: An array of IPv6 cidr block association set information.
            returned: always
            type: complex
            contains:
                association_id:
                    description: The association ID.
                    returned: always
                    type: str
                ipv6_cidr_block:
                    description: The IPv6 CIDR block that is associated with the subnet.
                    returned: always
                    type: str
                ipv6_cidr_block_state:
                    description: A hash/dict that contains a single item. The state of the cidr block association.
                    returned: always
                    type: dict
                    contains:
                        state:
                            description: The CIDR block association state.
                            returned: always
                            type: str
        map_customer_owned_ip_on_launch:
            description:
            - Indicates whether a network interface receives a customer-owned IPv4 address.
            type: bool
            sample: flase
        owner_id:
            description: The ID of the Amazon Web Services account that owns the subnet.
            type: str
            sample: 12344567
        private_dns_name_options_on_launch:
            description:
            - The type of hostnames to assign to instances in the subnet at launch.
            - An instance hostname is based on the IPv4 address or ID of the instance.
            type: dict
            sample: {
                "enable_resource_name_dns_a_record": false,
                "enable_resource_name_dns_aaaa_record": false,
                "hostname_type": "ip-name"
            }
        subnet_arn:
            description: The Amazon Resource Name (ARN) of the subnet.
            type: str
            sample: arn:aws:ec2:us-east-1:xxx:subnet/subnet-xxx
        subnet_id:
            description: The ID of the Subnet.
            returned: always
            type: str
"""

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_subnets
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


def _describe_subnets(connection, module: AnsibleAWSModule) -> None:
    """
    Describe Subnets.

    module  : AnsibleAWSModule object
    connection  : boto3 client connection object
    """
    # collect parameters
    filters = ansible_dict_to_boto3_filter_list(module.params.get("filters"))
    subnet_ids = module.params.get("subnet_ids")

    if subnet_ids is None:
        # Set subnet_ids to empty list if it is None
        subnet_ids = []

    # init empty list for return vars
    subnet_info = list()

    # Get the basic VPC info
    subnets = describe_subnets(connection, SubnetIds=subnet_ids, Filters=filters)

    for subnet in subnets:
        # for backwards compatibility
        subnet["id"] = subnet["SubnetId"]
        subnet_info.append(camel_dict_to_snake_dict(subnet))
        # convert tag list to ansible dict
        subnet_info[-1]["tags"] = boto3_tag_list_to_ansible_dict(subnet.get("Tags", []))

    module.exit_json(subnets=subnet_info)


def main() -> None:
    argument_spec = dict(
        subnet_ids=dict(type="list", elements="str", default=[], aliases=["subnet_id"]),
        filters=dict(type="dict", default={}),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    connection = module.client("ec2")

    _describe_subnets(connection, module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Tim Cragg (@timcragg)
# Copyright: (c) 2025, Shreyas Srish (@shrsr)
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_l4l7_device_selection_interface_context
version_added: "2.12.0"
short_description: Manage L4-L7 Device Selection Policy Logical Interface Contexts (vns:LIfCtx)
description:
- Manage L4-L7 Device Selection Policy Logical Interface Contexts
options:
  tenant:
    description:
    - The name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  contract:
    description:
    - The name of an existing contract.
    type: str
    aliases: [ contract_name ]
  graph:
    description:
    - The name of an existing Service Graph Template.
    type: str
    aliases: [ service_graph, service_graph_name ]
  node:
    description:
    - The name of an existing Service Graph Node.
    type: str
    aliases: [ node_name ]
  context:
    description:
    - The name of the logical interface context.
    type: str
  l3_destination:
    description:
    - Whether the context is a Layer3 destination.
    - The APIC defaults to C(true) when unset during creation.
    type: bool
    aliases: [ l3_dest ]
  permit_log:
    description:
    - Whether to log permitted traffic.
    - The APIC defaults to C(false) when unset during creation.
    type: bool
  bridge_domain:
    description:
    - The Bridge Domain to bind to the Context.
    type: str
    aliases: [ bd, bd_name ]
  bridge_domain_tenant:
    description:
    - The tenant the Bridge Domain resides in.
    - Omit this variable if both context and Bridge Domain are in the same tenant.
    - Intended use case is for when the Bridge Domain is in the common tenant, but the context is not.
    type: str
    aliases: [ bd_tenant ]
  logical_device:
    description:
    - The Logical Device to bind the context to.
    type: str
  logical_interface:
    description:
    - The Logical Interface to bind the context to.
    type: str
  redirect_policy:
    description:
    - The Redirect Policy to bind the context to.
    type: str
  permit_handoff:
    description:
    - Indicates whether to allow handoff of traffic to the associated logical interface.
    - The APIC defaults to C(false) when unset during creation.
    type: bool
  acl:
    description:
    - Specifies whether an Access Control List (ACL) is applied to the logical interface.
    - The APIC defaults to C(false) when unset during creation.
    type: bool
  description:
    description:
    - A brief description for the Logical Interface Context.
    type: str
  rule_type:
    description:
    - Indicates whether the context uses a specific rule type for traffic handling.
    - The APIC defaults to C(false) when unset during creation.
    type: bool
  l3out:
    description:
    - The name of the Layer 3 Outside (L3Out) object.
    type: str
  l3out_tenant:
    description:
    - The tenant in which the L3Out resides.
    - Omit this variable if both context and L3Out are in the same tenant.
    type: str
  external_epg:
    description:
    - The name of the External Network Instance Profile (External EPG or ExtEpg) associated with the L3Out.
    type: str
  redistribute:
    description:
    - A list of routing protocols whose routes should be redistributed.
    type: list
    elements: str
    choices: [ bgp, ospf, connected, static ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
- The I(tenant), I(graph), I(contract) and I(node) must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant), M(cisco.aci.aci_l4l7_service_graph_template), M(cisco.aci.aci_contract)
  and M(cisco.aci.aci_l4l7_service_graph_template_node) modules can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- module: cisco.aci.aci_l4l7_service_graph_template
- module: cisco.aci.aci_contract
- module: cisco.aci.aci_l4l7_service_graph_template_node
- name: APIC Management Information Model reference
  description: More information about the internal APIC class, B(vns:LIfCtx)
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Tim Cragg (@timcragg)
- Shreyas Srish (@shrsr)
"""

EXAMPLES = r"""
- name: Add a new interface context
  cisco.aci.aci_l4l7_device_selection_interface_context:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    contract: my_contract
    graph: my_graph
    node: my_node
    context: provider
    state: present
  delegate_to: localhost

- name: Query an interface context
  cisco.aci.aci_l4l7_device_selection_interface_context:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    contract: my_contract
    graph: my_graph
    node: my_node
    context: consumer
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all interface contexts
  cisco.aci.aci_l4l7_device_selection_interface_context:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Delete an interface context
  cisco.aci.aci_l4l7_device_selection_interface_context:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    contract: my_contract
    graph: my_graph
    node: my_node
    context: provider
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),
        contract=dict(type="str", aliases=["contract_name"]),
        graph=dict(type="str", aliases=["service_graph", "service_graph_name"]),
        node=dict(type="str", aliases=["node_name"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        context=dict(type="str"),
        l3_destination=dict(type="bool", aliases=["l3_dest"]),
        permit_log=dict(type="bool"),
        bridge_domain=dict(type="str", aliases=["bd", "bd_name"]),
        bridge_domain_tenant=dict(type="str", aliases=["bd_tenant"]),
        logical_device=dict(type="str"),
        logical_interface=dict(type="str"),
        redirect_policy=dict(type="str"),
        permit_handoff=dict(type="bool"),
        acl=dict(type="bool"),
        description=dict(type="str"),
        rule_type=dict(type="bool"),
        l3out=dict(type="str"),
        l3out_tenant=dict(type="str"),
        external_epg=dict(type="str"),
        redistribute=dict(type="list", elements="str", choices=["bgp", "ospf", "connected", "static"]),
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "contract", "graph", "node", "context"]],
            ["state", "present", ["tenant", "contract", "graph", "node", "context"]],
        ],
        required_together=[
            ("l3out", "external_epg"),
        ],
        mutually_exclusive=[
            ("bridge_domain", "l3out"),
            ("bridge_domain", "l3out_tenant"),
            ("bridge_domain", "external_epg"),
            ("bridge_domain", "redistribute"),
            ("l3out", "bridge_domain_tenant"),
        ],
    )

    aci = ACIModule(module)

    tenant = module.params.get("tenant")
    state = module.params.get("state")
    contract = module.params.get("contract")
    graph = module.params.get("graph")
    node = module.params.get("node")
    context = module.params.get("context")
    l3_destination = aci.boolean(module.params.get("l3_destination"))
    permit_log = aci.boolean(module.params.get("permit_log"))
    bridge_domain = module.params.get("bridge_domain")
    bridge_domain_tenant = module.params.get("bridge_domain_tenant")
    l3out = module.params.get("l3out")
    l3out_tenant = module.params.get("l3out_tenant")
    external_epg = module.params.get("external_epg")
    redistribute = module.params.get("redistribute")
    logical_device = module.params.get("logical_device")
    logical_interface = module.params.get("logical_interface")
    redirect_policy = module.params.get("redirect_policy")
    permit_handoff = aci.boolean(module.params.get("permit_handoff"))
    acl = aci.boolean(module.params.get("acl"))
    description = module.params.get("description")
    rule_type = aci.boolean(module.params.get("rule_type"))

    ldev_ctx_rn = "ldevCtx-c-{0}-g-{1}-n-{2}".format(contract, graph, node) if (contract, graph, node) != (None, None, None) else None

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="vnsLDevCtx",
            aci_rn=ldev_ctx_rn,
            module_object=ldev_ctx_rn,
            target_filter={"dn": ldev_ctx_rn},
        ),
        subclass_2=dict(
            aci_class="vnsLIfCtx",
            aci_rn="lIfCtx-c-{0}".format(context),
            module_object=context,
            target_filter={"connNameOrLbl": context},
        ),
        child_classes=["vnsRsLIfCtxToBD", "vnsRsLIfCtxToLIf", "vnsRsLIfCtxToSvcRedirectPol", "vnsRsLIfCtxToInstP"],
    )

    aci.get_existing()

    if state == "present":
        child_configs = []
        if bridge_domain:
            if bridge_domain_tenant is None:
                bridge_domain_tenant = tenant
            bd_tdn = "uni/tn-{0}/BD-{1}".format(bridge_domain_tenant, bridge_domain)
            child_configs.append({"vnsRsLIfCtxToBD": {"attributes": {"tDn": bd_tdn}}})
        else:
            bd_tdn = None
        if l3out:
            if l3out_tenant is None:
                l3out_tenant = tenant
            if redistribute is not None:
                redistribute = ",".join(redistribute)
            l3out_tdn = "uni/tn-{0}/out-{1}/instP-{2}".format(l3out_tenant, l3out, external_epg)
            child_configs.append({"vnsRsLIfCtxToInstP": {"attributes": {"tDn": l3out_tdn, "redistribute": redistribute}}})
        else:
            l3out_tdn = None
        if logical_interface:
            log_intf_tdn = "uni/tn-{0}/lDevVip-{1}/lIf-{2}".format(tenant, logical_device, logical_interface)
            child_configs.append({"vnsRsLIfCtxToLIf": {"attributes": {"tDn": log_intf_tdn}}})
        else:
            log_intf_tdn = None
        if redirect_policy:
            redir_pol_tdn = "uni/tn-{0}/svcCont/svcRedirectPol-{1}".format(tenant, redirect_policy)
            child_configs.append({"vnsRsLIfCtxToSvcRedirectPol": {"attributes": {"tDn": redir_pol_tdn}}})
        else:
            redir_pol_tdn = None
        # Validate if existing and remove child objects when do not match provided configuration
        if isinstance(aci.existing, list) and len(aci.existing) > 0:
            for child in aci.existing[0].get("vnsLIfCtx", {}).get("children", {}):
                if child.get("vnsRsLIfCtxToBD") and child.get("vnsRsLIfCtxToBD").get("attributes").get("tDn") != bd_tdn:
                    # Appending to child_config list not possible because of APIC Error 103: child (Rn) of class vnsRsLIfCtxToBD is already attached.
                    # A seperate delete request to dn of the vnsRsLIfCtxToBD is needed to remove the object prior to adding to child_configs.
                    # child_configs.append(
                    #     {
                    #         "vnsRsLIfCtxToBD": {
                    #             "attributes": {
                    #                 "dn": child.get("vnsRsLIfCtxToBD").get("attributes").get("dn"),
                    #                 "status": "deleted",
                    #             }
                    #         }
                    #     }
                    # )
                    aci.api_call(
                        "DELETE",
                        "{0}/api/mo/uni/tn-{1}/ldevCtx-c-{2}-g-{3}-n-{4}/lIfCtx-c-{5}/rsLIfCtxToBD.json".format(
                            aci.base_url, tenant, contract, graph, node, context
                        ),
                    )
                elif child.get("vnsRsLIfCtxToInstP") and child.get("vnsRsLIfCtxToInstP").get("attributes").get("tDn") != l3out_tdn:
                    # Appending to child_config list not possible because of APIC Error 103: child (Rn) of class vnsRsLIfCtxToInstP is already attached.
                    # A seperate delete request to dn of the vnsRsLIfCtxToInstP is needed to remove the object prior to adding to child_configs.
                    aci.api_call(
                        "DELETE",
                        "{0}/api/mo/uni/tn-{1}/ldevCtx-c-{2}-g-{3}-n-{4}/lIfCtx-c-{5}/rsLIfCtxToInstP.json".format(
                            aci.base_url, tenant, contract, graph, node, context
                        ),
                    )
                elif child.get("vnsRsLIfCtxToLIf") and child.get("vnsRsLIfCtxToLIf").get("attributes").get("tDn") != log_intf_tdn:
                    child_configs.append(
                        {
                            "vnsRsLIfCtxToLIf": {
                                "attributes": {
                                    "dn": child.get("vnsRsLIfCtxToLIf").get("attributes").get("dn"),
                                    "status": "deleted",
                                }
                            }
                        }
                    )
                elif child.get("vnsRsLIfCtxToSvcRedirectPol") and child.get("vnsRsLIfCtxToSvcRedirectPol").get("attributes").get("tDn") != redir_pol_tdn:
                    child_configs.append(
                        {
                            "vnsRsLIfCtxToSvcRedirectPol": {
                                "attributes": {
                                    "dn": child.get("vnsRsLIfCtxToSvcRedirectPol").get("attributes").get("dn"),
                                    "status": "deleted",
                                }
                            }
                        }
                    )
        aci.payload(
            aci_class="vnsLIfCtx",
            class_config=dict(
                connNameOrLbl=context,
                l3Dest=l3_destination,
                permitLog=permit_log,
                permitHandoff=permit_handoff,
                acl=acl,
                descr=description,
                ruleType=rule_type,
            ),
            child_configs=child_configs,
        )
        aci.get_diff(aci_class="vnsLIfCtx")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2026 Zun Yang Chin, (@zunyangc)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_storagesharedirectory
version_added: "3.19.0"
short_description: Manage directories inside an Azure file share
description:
    - Create, update or delete a directory (folder) inside an Azure Storage file share.
options:
    resource_group:
        description:
            - Name of the resource group that contains the storage account.
        required: true
        type: str
    account_name:
        description:
            - Name of the parent storage account hosting the file share.
        required: true
        type: str
    share_name:
        description:
            - Name of the existing file share.
        required: true
        type: str
    name:
        description:
            - Path of the directory to manage inside the file share, for example C(foo) or C(foo/bar/baz).
            - When I(parents=false), the default, the parent path must already exist.
        required: true
        type: str
        aliases:
            - path
    metadata:
        description:
            - A name-value pair to associate with the directory as metadata.
            - All existing metadata will be replaced on each update.
        type: dict
    parents:
        description:
            - When C(true) and C(state=present), missing intermediate directories in C(name) are
              created.
            - When C(false), the default, the module fails if any parent segment does not exist.
        type: bool
        default: false
    force:
        description:
            - When C(true) and C(state=absent), recursively delete the contents of the directory
              before deleting the directory itself.
            - When C(false), the default, the module fails if the directory is not empty.
        type: bool
        default: false
    state:
        description:
            - State of the directory. Use C(present) to create or update and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Zun Yang Chin (@zunyangc)
'''

EXAMPLES = '''
---
- name: Create a directory at the share root
  azure.azcollection.azure_rm_storagesharedirectory:
    resource_group: myResourceGroup
    account_name: myStorageAccount
    share_name: myshare
    name: parent

- name: Create a nested directory tree (mkdir -p semantics)
  azure.azcollection.azure_rm_storagesharedirectory:
    resource_group: myResourceGroup
    account_name: myStorageAccount
    share_name: myshare
    name: parent/child/grandchild
    parents: true
    metadata:
      owner: ansible

- name: Update metadata on an existing directory
  azure.azcollection.azure_rm_storagesharedirectory:
    resource_group: myResourceGroup
    account_name: myStorageAccount
    share_name: myshare
    name: parent
    metadata:
      owner: ansible
      env: prod

- name: Delete an empty directory
  azure.azcollection.azure_rm_storagesharedirectory:
    resource_group: myResourceGroup
    account_name: myStorageAccount
    share_name: myshare
    name: parent/child/grandchild
    state: absent

- name: Delete a directory recursively (including children)
  azure.azcollection.azure_rm_storagesharedirectory:
    resource_group: myResourceGroup
    account_name: myStorageAccount
    share_name: myshare
    name: parent
    state: absent
    force: true
'''

RETURN = '''
state:
    description:
        - Facts about the current state of the directory.
    returned: always
    type: complex
    contains:
        name:
            description:
                - Name of the directory (the leaf segment of the path).
            returned: always
            type: str
            sample: grandchild
        path:
            description:
                - Full path of the directory within the file share.
            returned: always
            type: str
            sample: parent/child/grandchild
        share:
            description:
                - Name of the parent file share.
            returned: always
            type: str
            sample: myshare
        account:
            description:
                - Name of the parent storage account.
            returned: always
            type: str
            sample: myStorageAccount
        etag:
            description:
                - The directory entity tag.
            returned: when the directory exists
            type: str
            sample: '"0x8D75E4BA3E275F1"'
        last_modified:
            description:
                - The time the directory was last modified.
            returned: when the directory exists
            type: str
            sample: '2026-05-14T03:11:22+00:00'
        metadata:
            description:
                - Name-value pairs associated with the directory as metadata.
            returned: when the directory exists
            type: dict
            sample: {"owner": "ansible"}
'''

try:
    from azure.core.exceptions import ResourceNotFoundError, ResourceExistsError
except ImportError:
    # Handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class AzureRMStorageShareDirectory(AzureRMModuleBase):
    '''
    Manage a directory inside an Azure Storage file share via the data plane.
    '''

    def __init__(self):

        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            account_name=dict(type='str', required=True),
            share_name=dict(type='str', required=True),
            name=dict(type='str', required=True, aliases=['path']),
            metadata=dict(type='dict'),
            parents=dict(type='bool', default=False),
            force=dict(type='bool', default=False),
            state=dict(type='str', default='present', choices=['present', 'absent']),
        )
        self.results = dict(
            changed=False,
            state=dict(),
        )

        self.resource_group = None
        self.account_name = None
        self.share_name = None
        self.name = None
        self.metadata = None
        self.parents = False
        self.force = False
        self.state = None

        super(AzureRMStorageShareDirectory, self).__init__(
            self.module_arg_spec,
            supports_check_mode=True,
            supports_tags=False,
        )

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        # Normalise the path (no leading/trailing slashes).
        self.name = self.name.strip('/').strip()
        if not self.name:
            self.fail("The directory 'name' must not be empty.")

        directory_client = self._get_directory_client(self.name)
        existing = self._get_directory(directory_client)

        if self.state == 'present':
            if existing is None:
                self.results['changed'] = True
                if not self.check_mode:
                    self._create_directory(directory_client)
                    if self.metadata is not None:
                        directory_client.set_directory_metadata(metadata=self.metadata)
                    self.results['state'] = self._build_state(self._get_directory(directory_client))
                else:
                    self.results['state'] = self._build_state(None)
            else:
                if self.metadata is not None and (existing.get('metadata') or {}) != self.metadata:
                    self.results['changed'] = True
                    if not self.check_mode:
                        directory_client.set_directory_metadata(metadata=self.metadata)
                        existing = self._get_directory(directory_client)
                self.results['state'] = self._build_state(existing)
        else:
            if existing is not None:
                self.results['changed'] = True
                if not self.check_mode:
                    self._delete_directory(directory_client)
                self.results['state'] = self._build_state(None)
            else:
                self.results['state'] = self._build_state(None)

        return self.results

    def _get_directory_client(self, directory_path):
        return self.get_share_directory_client(resource_group_name=self.resource_group,
                                               storage_account_name=self.account_name,
                                               share_name=self.share_name,
                                               directory_path=directory_path)

    def _get_file_client(self, file_path):
        return self.get_share_file_client(resource_group_name=self.resource_group,
                                          storage_account_name=self.account_name,
                                          share_name=self.share_name,
                                          file_path=file_path)

    def _get_directory(self, directory_client):
        try:
            props = directory_client.get_directory_properties()
        except ResourceNotFoundError:
            return None
        except Exception as exc:
            self.fail("Error retrieving directory '{0}' on share '{1}': {2}".format(
                directory_client.directory_path, self.share_name, str(exc)))

        etag = getattr(props, 'etag', None)
        last_modified = getattr(props, 'last_modified', None)
        return dict(
            metadata=getattr(props, 'metadata', None) or {},
            etag=etag.replace('"', '') if isinstance(etag, str) else etag,
            last_modified=str(last_modified) if last_modified is not None else None,
        )

    def _create_directory(self, directory_client):
        if self.parents:
            self._create_parents(self.name)
        try:
            directory_client.create_directory()
        except ResourceExistsError:
            # Race or pre-existing — treat as no-op.
            pass
        except Exception as exc:
            self.fail("Error creating directory '{0}' on share '{1}': {2}".format(
                self.name, self.share_name, str(exc)))

    def _create_parents(self, full_path):
        segments = full_path.split('/')
        accumulated = ''
        for segment in segments[:-1]:
            if not segment:
                continue
            accumulated = segment if not accumulated else accumulated + '/' + segment
            parent_client = self._get_directory_client(accumulated)
            try:
                parent_client.create_directory()
            except ResourceExistsError:
                continue
            except Exception as exc:
                self.fail("Error creating parent directory '{0}': {1}".format(accumulated, str(exc)))

    def _delete_directory(self, directory_client):
        if self.force:
            self._recursive_delete(directory_client)
            return
        try:
            directory_client.delete_directory()
        except Exception as exc:
            self.fail("Error deleting directory '{0}' on share '{1}': {2}".format(
                self.name, self.share_name, str(exc)))

    def _recursive_delete(self, directory_client):
        '''
        Walk the directory client-side, deleting every file and subdirectory before
        deleting the target directory itself. The REST API has no recursive delete.
        '''
        try:
            entries = list(directory_client.list_directories_and_files())
        except Exception as exc:
            self.fail("Error listing contents of directory '{0}': {1}".format(
                directory_client.directory_path, str(exc)))

        for entry in entries:
            child_path = directory_client.directory_path + '/' + entry['name']
            if entry.get('is_directory'):
                child_client = self._get_directory_client(child_path)
                self._recursive_delete(child_client)
            else:
                file_client = self._get_file_client(child_path)
                try:
                    file_client.delete_file()
                except Exception as exc:
                    self.fail("Error deleting file '{0}': {1}".format(child_path, str(exc)))

        try:
            directory_client.delete_directory()
        except Exception as exc:
            self.fail("Error deleting directory '{0}' on share '{1}': {2}".format(
                directory_client.directory_path, self.share_name, str(exc)))

    def _build_state(self, props):
        leaf = self.name.split('/')[-1]
        state = dict(
            name=leaf,
            path=self.name,
            share=self.share_name,
            account=self.account_name,
        )
        if props is not None:
            state.update(dict(
                etag=props.get('etag'),
                last_modified=props.get('last_modified'),
                metadata=props.get('metadata') or {},
            ))
        return state


def main():
    AzureRMStorageShareDirectory()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

import random
import re
import string

from ansible.module_utils.common.text.converters import to_bytes
from ansible.module_utils.common.text.converters import to_text
from ansible.plugins.shell.powershell import _common_args


class TerminalManager:
    def __init__(self, connection):
        self.connection = connection

    def prepare_terminal(self) -> None:
        """perform any one-time terminal settings"""
        # No Windows setup for now
        if self.connection.is_windows:
            return

        # Ensure SSM Session has started
        self.ensure_ssm_session_has_started()

        # Disable echo command
        self.disable_echo_command()  # pylint: disable=unreachable

        # Disable prompt command
        self.disable_prompt_command()  # pylint: disable=unreachable

        self.connection.verbosity_display(4, "PRE Terminal configured")  # pylint: disable=unreachable

    def wrap_command(self, cmd: str, mark_start: str, mark_end: str) -> str:
        """Wrap command so stdout and status can be extracted"""

        if self.connection.is_windows:
            if not cmd.startswith(" ".join(_common_args) + " -EncodedCommand"):
                cmd = self.connection._shell._encode_script(cmd, preserve_rc=True)
            cmd = f"{cmd}; echo {mark_start}\necho {mark_end}\n"
        else:
            cmd = (
                f"printf '%s\\n' '{mark_start}';\n"
                f"echo | {cmd};\n"
                f"printf '\\n%s\\n%s\\n' \"$?\" '{mark_end}';\n"
            )  # fmt: skip

        self.connection.verbosity_display(4, f"wrap_command: \n'{to_text(cmd)}'")
        return cmd

    def disable_echo_command(self) -> None:
        """Disable echo command from the host"""
        disable_echo_cmd = to_bytes("stty -echo\n", errors="surrogate_or_strict")

        # Send command
        self.connection.verbosity_display(4, f"DISABLE ECHO Disabling Prompt: \n{disable_echo_cmd}")
        self.connection.session_manager.stdin_write(disable_echo_cmd)

        self.connection.session_manager.wait_for_match(label="DISABLE ECHO", cmd=disable_echo_cmd, match="stty -echo")

    def disable_prompt_command(self) -> None:
        """Disable prompt command from the host"""
        end_mark = "".join([random.choice(string.ascii_letters) for i in range(self.connection.MARK_LENGTH)])
        disable_prompt_cmd = to_bytes(
            "PS1='' ; bind 'set enable-bracketed-paste off'; printf '\\n%s\\n' '" + end_mark + "'\n",
            errors="surrogate_or_strict",
        )
        disable_prompt_reply = re.compile(r"\r\r\n" + re.escape(end_mark) + r"\r\r\n", re.MULTILINE)

        # Send command
        self.connection.verbosity_display(4, f"DISABLE PROMPT Disabling Prompt: \n{disable_prompt_cmd}")
        self.connection.session_manager.stdin_write(disable_prompt_cmd)

        self.connection.session_manager.wait_for_match(
            label="DISABLE PROMPT", cmd=disable_prompt_cmd, match=disable_prompt_reply.search
        )

    def ensure_ssm_session_has_started(self) -> None:
        """Ensure the SSM session has started on the host. We poll stdout
        until we match the following string 'Starting session with SessionId'
        """
        self.connection.session_manager.wait_for_match(
            label="START SSM SESSION", cmd="start_session", match="Starting session with SessionId"
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_devices_packet_capture_schedules
short_description: Resource module for organizations _devices _packet _capture _schedules
description:
  - Manage operations create, update and delete of the resource organizations _devices
    _packet _capture _schedules.
  - Create a schedule for packet capture.
  - Delete schedule from cloud.
  - Update a schedule for packet capture.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  devices:
    description: Device details.
    elements: dict
    suboptions:
      interface:
        description: Interface to capture.
        type: str
      serial:
        description: Serial of the devices to schedule packet capture.
        type: str
      switchports:
        description: Switchports to capture.
        type: str
    type: list
  duration:
    description: Duration of the capture in seconds.
    type: int
  enabled:
    description: Enable or disable the schedule.
    type: bool
  filterExpression:
    description: Filter expression for the capture.
    type: str
  name:
    description: Name of the packet capture file.
    type: str
  notes:
    description: Reason for capture.
    type: str
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  schedule:
    description: Schedule details.
    suboptions:
      endTs:
        description: End date and time of the recurring schedule entry.
        type: str
      frequency:
        description: Frequency of the recurring schedule entry (hour, week, month,
          day, minute).
        type: str
      name:
        description: Name of the schedule.
        type: str
      recurrence:
        description: Cardinality of the schedule frequency, ex. 1 = every day, 2 =
          every other day (when frequency = day).
        type: int
      startTs:
        description: Start date and time of the recurring schedule entry.
        type: str
      weekdays:
        description: Weekdays for the schedule Sunday, Monday, Tuesday, Wednesday,
          Thursday, Friday, Saturday.
        elements: str
        type: list
    type: dict
  scheduleId:
    description: ScheduleId path parameter. Schedule ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations createOrganizationDevicesPacketCaptureSchedule
    description: Complete reference of the createOrganizationDevicesPacketCaptureSchedule
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-organization-devices-packet-capture-schedule
  - name: Cisco Meraki documentation for organizations deleteOrganizationDevicesPacketCaptureSchedule
    description: Complete reference of the deleteOrganizationDevicesPacketCaptureSchedule
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-organization-devices-packet-capture-schedule
  - name: Cisco Meraki documentation for organizations updateOrganizationDevicesPacketCaptureSchedule
    description: Complete reference of the updateOrganizationDevicesPacketCaptureSchedule
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-organization-devices-packet-capture-schedule
notes:
  - SDK Method used are
    organizations.Organizations.create_organization_devices_packet_capture_schedule,
    organizations.Organizations.delete_organization_devices_packet_capture_schedule,
    organizations.Organizations.update_organization_devices_packet_capture_schedule,
  - Paths used are
    post /organizations/{organizationId}/devices/packetCapture/schedules,
    delete /organizations/{organizationId}/devices/packetCapture/schedules/{scheduleId},
    put /organizations/{organizationId}/devices/packetCapture/schedules/{scheduleId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_devices_packet_capture_schedules:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    devices:
      - interface: TenGigabitEthernet0/0/0
        serial: Q234-ABCD-5678
        switchports: 1, 2
    duration: 60
    enabled: true
    filterExpression: (icmp)
    name: daily_capture_for_debugging
    notes: Debugging persistent issue on device
    organizationId: string
    schedule:
      endTs: '2021-01-01T14:00:00Z'
      frequency: daily
      name: Daily at 1pm
      recurrence: 1
      startTs: '2021-01-01T13:00:00Z'
      weekdays:
        - Monday
        - Wednesday
        - Friday
- name: Update by id
  cisco.meraki.organizations_devices_packet_capture_schedules:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    devices:
      - interface: TenGigabitEthernet0/0/0
        serial: Q234-ABCD-5678
        switchports: 1, 2
    duration: 60
    enabled: true
    filterExpression: (icmp)
    name: daily_capture_for_debugging
    notes: Debugging persistent issue on device
    organizationId: string
    schedule:
      endTs: '2021-01-01T14:00:00Z'
      frequency: daily
      name: Daily at 1pm
      recurrence: 1
      startTs: '2021-01-01T13:00:00Z'
      weekdays:
        - Monday
        - Wednesday
        - Friday
    scheduleId: string
- name: Delete by id
  cisco.meraki.organizations_devices_packet_capture_schedules:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    organizationId: string
    scheduleId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "scheduleId": "string",
      "devices": [
        {
          "device": {
            "serial": "string",
            "switchports": "string",
            "interface": "string"
          }
        }
      ],
      "name": "string",
      "admin": {
        "id": "string",
        "name": "string"
      },
      "notes": "string",
      "duration": 0,
      "filterExpression": "string",
      "createdAt": "string",
      "updatedAt": "string",
      "captureCount": 0,
      "lastCaptureId": "string",
      "enabled": true,
      "priority": 0,
      "schedule": {
        "name": "string",
        "startTs": "string",
        "endTs": "string",
        "frequency": "string",
        "weekdays": [
          "string"
        ],
        "recurrence": 0,
        "nextCaptureTs": "string"
      },
      "warnings": [
        "string"
      ]
    }
"""
